# -*- coding: utf-8 -*-
import unittest
import __builtin__
import StringIO
import uuid

import mock
from tutum.api.exceptions import *
from tutumcli.commands import *
import tutumcli


class LoginTestCase(unittest.TestCase):
    def setUp(self):
        # backup configfile
        self.data = None
        file = None
        try:
            configFile = os.path.join(os.path.expanduser('~'), TUTUM_FILE)
            file = open(configFile, 'r')
            self.data = file.read()
        except:
            pass
        finally:
            if file:
                file.close()

        self.raw_input_holder = __builtin__.raw_input
        self.stdout = sys.stdout
        self.stderr = sys.stderr
        sys.stdout = self.stdout_buf = StringIO.StringIO()
        sys.stderr = self.stderr_buf = StringIO.StringIO()

    def tearDown(self):
        __builtin__.raw_input = self.raw_input_holder
        sys.stdout = self.stdout
        sys.stdout = self.stderr

        if self.data:
            file = None
            try:
                configFile = os.path.join(os.path.expanduser('~'), TUTUM_FILE)
                file = open(configFile, 'w')
                file.write(self.data)
            except:
                pass
            finally:
                if file:
                    file.close()

    def set_username(self, username):
        __builtin__.raw_input = lambda _: username

    @mock.patch('tutumcli.commands.getpass.getpass', return_value='test_password')
    @mock.patch('tutumcli.commands.tutum.auth.get_auth')
    def test_login_success(self, mock_get_auth, mock_password):
        user = uuid.uuid4()
        apikey = uuid.uuid4()
        __builtin__.raw_input = lambda _: user  # set username
        mock_get_auth.return_value = (user, apikey)
        login()
        out = self.stdout_buf.getvalue().strip()
        self.stdout_buf.truncate(0)
        self.assertEqual('Login succeeded!', out)
        configFile = os.path.join(os.path.expanduser('~'), TUTUM_FILE)
        output = '''[auth]
user = %s
apikey = %s''' % (user, apikey)
        file = open(configFile, 'r')
        try:
            data = file.read()
            self.assertEqual(output.strip(), data.strip())
        finally:
            file.close()
            os.remove(configFile)

    @mock.patch('tutumcli.commands.utils.try_register', return_value=(True, 'Registration succeeded!'))
    @mock.patch('tutumcli.commands.getpass.getpass', return_value='test_password')
    @mock.patch('tutumcli.commands.tutum.auth.get_auth', side_effect=TutumAuthError)
    def test_login_register_success(self, mock_get_auth, mock_getpass, mock_register):
        __builtin__.raw_input = lambda _: 'test_username'  # set username
        login()
        mock_register.assert_called_with('test_username', 'test_password')
        out = self.stdout_buf.getvalue().strip()
        self.stdout_buf.truncate(0)
        self.assertEqual('Registration succeeded!', out)

    @mock.patch('tutumcli.commands.utils.try_register',
                return_value=(False, 'ERROR: username: A user with that username already exists.'))
    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.getpass.getpass', return_value='test_password')
    @mock.patch('tutumcli.commands.tutum.auth.get_auth', side_effect=TutumAuthError)
    def test_login_register_user_exist(self, mock_get_auth, mock_getpass, mock_exit, mock_register):
        __builtin__.raw_input = lambda _: 'test_username'  # set username
        login()
        mock_register.assert_called_with('test_username', 'test_password')
        out = self.stderr_buf.getvalue().strip()
        self.stderr_buf.truncate(0)
        self.assertEqual('Wrong username and/or password. Please try to login again', out)
        mock_exit.assert_called_with(TUTUM_AUTH_ERROR_EXIT_CODE)

    @mock.patch('tutumcli.commands.utils.try_register',
                return_value=(False, 'password1: This field is required.\npassword2: This field is required.'
                                     '\nemail: This field is required.'))
    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.getpass.getpass', return_value='test_password')
    @mock.patch('tutumcli.commands.tutum.auth.get_auth', side_effect=TutumAuthError)
    def test_login_register_password_required(self, mock_get_auth, mock_getpass, mock_exit, mock_register):
        __builtin__.raw_input = lambda _: 'test_username'  # set username
        login()
        mock_register.assert_called_with('test_username', 'test_password')
        out = self.stderr_buf.getvalue().strip()
        self.stderr_buf.truncate(0)
        self.assertEqual('password: This field is required.\nemail: This field is required.', out)
        mock_exit.assert_called_with(TUTUM_AUTH_ERROR_EXIT_CODE)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.getpass.getpass', return_value='test_password')
    @mock.patch('tutumcli.commands.tutum.auth.get_auth', side_effect=Exception('Cannot open config file'))
    def test_login_register_Exception(self, mock_get_auth, mock_getpass, mock_exit):
        __builtin__.raw_input = lambda _: 'test_username'  # set username
        login()
        out = self.stderr_buf.getvalue().strip()
        self.stderr_buf.truncate(0)
        self.assertEqual('Cannot open config file', out)
        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceCreateTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.start')
    @mock.patch('tutumcli.commands.tutum.Service.save')
    @mock.patch('tutumcli.commands.tutum.Service.create')
    def test_service_create(self, mock_create, mock_save, mock_start):
        exposed_ports = [800, 222]
        published_ports = ['80:80/tcp', '22:22']
        ports = utils.parse_published_ports(published_ports)
        ports.extend(utils.parse_exposed_ports(exposed_ports))
        container_envvars = ['MYSQL_ADMIN=admin', 'MYSQL_PASS=password']
        linked_to_service = ['mysql:mysql', 'redis:redis']

        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_create.return_value = service
        service_create('imagename', 'containername', 1, '256M', True, 3, '-d', '/bin/mysql',
                       exposed_ports, published_ports, container_envvars, [], '', linked_to_service,
                       'OFF', 'OFF', 'OFF', 'poweruser', True, None, None, None)

        mock_create.assert_called_with(image='imagename', name='containername', cpu_shares=1,
                                       memory='256M', privileged=True,
                                       target_num_containers=3, run_command='-d',
                                       entrypoint='/bin/mysql', container_ports=ports,
                                       container_envvars=utils.parse_envvars(container_envvars, []),
                                       linked_to_service=utils.parse_links(linked_to_service, 'to_service'),
                                       autorestart='OFF', autodestroy='OFF', autoredeploy='OFF',
                                       roles='poweruser', sequential_deployment=True, tags=[], bindings=[],
                                       deployment_strategy=None)
        mock_save.assert_called()
        mock_start.assert_not_called()
        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.Service.start')
    @mock.patch('tutumcli.commands.tutum.Service.save')
    @mock.patch('tutumcli.commands.tutum.Service.create')
    def test_service_create_exposing_publishing_same_port(self, mock_create, mock_save, mock_start):
        exposed_ports = [80]
        published_ports = ['800:80/tcp']
        ports = [{'inner_port': '80', 'outer_port': '800', 'protocol': 'tcp', 'published': True}]
        container_envvars = ['MYSQL_ADMIN=admin', 'MYSQL_PASS=password']
        linked_to_service = ['mysql:mysql', 'redis:redis']
        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_create.return_value = service
        service_run('imagename', 'containername', 1, '256M', True, 3, '-d', '/bin/mysql',
                    exposed_ports, published_ports, container_envvars, [], '', linked_to_service,
                    'OFF', 'OFF', 'OFF', 'poweruser', True, None, None, None)

        mock_create.assert_called_with(image='imagename', name='containername', cpu_shares=1,
                                       memory='256M', privileged=True,
                                       target_num_containers=3, run_command='-d',
                                       entrypoint='/bin/mysql', container_ports=ports,
                                       container_envvars=utils.parse_envvars(container_envvars, []),
                                       linked_to_service=utils.parse_links(linked_to_service, 'to_service'),
                                       autorestart='OFF', autodestroy='OFF', autoredeploy='OFF',
                                       roles='poweruser', sequential_deployment=True, tags=[], bindings=[],
                                       deployment_strategy=None)
        mock_save.assert_called()
        mock_start.assert_not_called()
        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Service.create', side_effect=TutumApiError)
    def test_service_create_with_exception(self, mock_create, mock_exit):
        exposed_ports = ['80', '22']
        published_ports = ['80:80/tcp', '22:22']
        container_envvars = ['MYSQL_ADMIN=admin', 'MYSQL_PASS=password']
        linked_to_service = ['mysql:mysql', 'redis:redis']
        service_create('imagename', 'containername', 1, '256M', True, 3, '-d', '/bin/mysql',
                       exposed_ports, published_ports, container_envvars, [], '', linked_to_service,
                       'OFF', 'OFF', 'OFF', 'poweruser', True, None, None, None)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceInspectTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.get_all_attributes')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service')
    def test_service_inspect(self, mock_fetch_remote_service, mock_get_all_attributes):
        output = '''{
  "key": [
    {
      "name": "test",
      "id": "1"
    }
  ]
}'''
        uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        service = tutumcli.commands.tutum.Service()
        service.uuid = uuid
        mock_fetch_remote_service.return_value = service
        mock_get_all_attributes.return_value = {'key': [{'name': 'test', 'id': '1'}]}
        service_inspect(['test_id'])

        self.assertEqual(' '.join(output.split()), ' '.join(self.buf.getvalue().strip().split()))
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service', side_effect=TutumApiError)
    def test_service_inspect_with_exception(self, mock_fetch_remote_service, mock_exit):
        service = tutumcli.commands.tutum.Service()
        mock_fetch_remote_service.return_value = service
        service_inspect(['test_id', 'test_id2'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceLogsTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.utils.fetch_remote_service')
    def test_service_logs(self, mock_fetch_remote_service):
        log = 'Here is the log'
        service = tutumcli.commands.tutum.Service
        service.logs = log
        mock_fetch_remote_service.return_value = service
        service_logs(['test_id'])

        self.assertEqual(log, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service', side_effect=TutumApiError)
    def test_service_logs_with_exception(self, mock_fetch_remote_service, mock_exit):
        service = tutumcli.commands.tutum.Service()
        mock_fetch_remote_service.return_value = service
        service_logs(['test_id', 'test_id2'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServicePsTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

        service1 = tutumcli.commands.tutum.Service()
        service1.current_num_containers = 3
        service1.name = 'SERVICE1'
        service1.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        service1.image_name = 'test/service1'
        service1.web_public_dns = 'service1.io'
        service1.state = 'Running'
        service1.deployed_datetime = ''
        service1.synchronized = True
        service1.public_dns = "www.myhello1service.com"
        service1.stack = "/resource_uri/service1"
        service2 = tutumcli.commands.tutum.Service()
        service2.current_num_containers = 2
        service2.name = 'SERVICE2'
        service2.uuid = '8B4CFE51-03BB-42D6-825E-3B533888D8CD'
        service2.image_name = 'test/service2'
        service2.web_public_dns = 'service2.io'
        service2.state = 'Stopped'
        service2.deployed_datetime = ''
        service2.synchronized = True
        service2.public_dns = "www.myhello2service.com"
        service2.stack = "/resource_uri/service2"
        self.servicelist = [service1, service2]
        stack1 = tutumcli.commands.tutum.Stack()
        stack1.resource_uri = "/resource_uri/service1"
        stack1.name = "service1"
        stack2 = tutumcli.commands.tutum.Stack()
        stack2.resource_uri = "/resource_uri/service2"
        stack2.name = "service2"
        self.stacklist = [stack1, stack2]

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Stack.list')
    @mock.patch('tutumcli.commands.tutum.Service.list')
    def test_service_ps(self, mock_list, mock_stack):
        output = u'''NAME      UUID      STATUS       #CONTAINERS  IMAGE          DEPLOYED    PUBLIC DNS               STACK
SERVICE1  7A4CFE51  \u25b6 Running              3  test/service1              www.myhello1service.com  service1
SERVICE2  8B4CFE51  \u25fc Stopped              2  test/service2              www.myhello2service.com  service2'''
        mock_list.return_value = self.servicelist
        mock_stack.return_value = self.stacklist
        service_ps(False, 'Running', None)

        mock_list.assert_called_with(state='Running', stack=None)
        self.buf.getvalue().strip()
        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.Stack.list')
    @mock.patch('tutumcli.commands.tutum.Service.list')
    def test_service_ps_quiet(self, mock_list, mock_stack):
        output = '''7A4CFE51-03BB-42D6-825E-3B533888D8CD
8B4CFE51-03BB-42D6-825E-3B533888D8CD'''
        mock_stack.return_value = self.stacklist
        mock_list.return_value = self.servicelist
        service_ps(True, None, None)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Service.list', side_effect=TutumApiError)
    def test_service_ps_with_exception(self, mock_list, mock_exit):
        service_ps(False, None, None)
        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)

    @mock.patch('tutumcli.commands.tutum.Stack.list')
    @mock.patch('tutumcli.commands.tutum.Service.list')
    def test_service_ps_unsync(self, mock_list, mock_stack):
        output = u'''NAME      UUID      STATUS          #CONTAINERS  IMAGE          DEPLOYED    PUBLIC DNS               STACK
SERVICE1  7A4CFE51  \u25b6 Running(*)              3  test/service1              www.myhello1service.com  service1
SERVICE2  8B4CFE51  \u25fc Stopped                 2  test/service2              www.myhello2service.com  service2

(*) Please note that this service needs to be redeployed to have its configuration changes applied'''
        self.servicelist[0].synchronized = False
        mock_stack.return_value = self.stacklist
        mock_list.return_value = self.servicelist
        service_ps(False, 'Running', None)

        mock_list.assert_called_with(state='Running', stack=None)
        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)


class ServiceRunTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.start')
    @mock.patch('tutumcli.commands.tutum.Service.save')
    @mock.patch('tutumcli.commands.tutum.Service.create')
    def test_service_run(self, mock_create, mock_save, mock_start):
        exposed_ports = [800, 222]
        published_ports = ['80:80/tcp', '22:22']
        ports = utils.parse_published_ports(published_ports)
        ports.extend(utils.parse_exposed_ports(exposed_ports))
        container_envvars = ['MYSQL_ADMIN=admin', 'MYSQL_PASS=password']
        linked_to_service = ['mysql:mysql', 'redis:redis']

        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_create.return_value = service
        mock_start.return_value = True
        service_run('imagename', 'containername', 1, '256M', True, 3, '-d', '/bin/mysql',
                    exposed_ports, published_ports, container_envvars, [], '', linked_to_service,
                    'OFF', 'OFF', 'OFF', 'poweruser', True, None, None, None)

        mock_create.assert_called_with(image='imagename', name='containername', cpu_shares=1,
                                       memory='256M', privileged=True,
                                       target_num_containers=3, run_command='-d',
                                       entrypoint='/bin/mysql', container_ports=ports,
                                       container_envvars=utils.parse_envvars(container_envvars, []),
                                       linked_to_service=utils.parse_links(linked_to_service, 'to_service'),
                                       autorestart='OFF', autodestroy='OFF', autoredeploy='OFF',
                                       roles='poweruser', sequential_deployment=True, tags=[], bindings=[],
                                       deployment_strategy=None)
        mock_save.assert_called()
        mock_start.assert_called()
        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.Service.start')
    @mock.patch('tutumcli.commands.tutum.Service.save')
    @mock.patch('tutumcli.commands.tutum.Service.create')
    def test_service_run_exposing_publishing_same_port(self, mock_create, mock_save, mock_start):
        exposed_ports = [80]
        published_ports = ['800:80/tcp']
        ports = [{'inner_port': '80', 'outer_port': '800', 'protocol': 'tcp', 'published': True}]
        container_envvars = ['MYSQL_ADMIN=admin', 'MYSQL_PASS=password']
        linked_to_service = ['mysql:mysql', 'redis:redis']

        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_create.return_value = service
        mock_start.return_value = True
        service_run('imagename', 'containername', 1, '256M', True, 3, '-d', '/bin/mysql',
                    exposed_ports, published_ports, container_envvars, [], '', linked_to_service,
                    'OFF', 'OFF', 'OFF', 'poweruser', True, None, None, None)

        mock_create.assert_called_with(image='imagename', name='containername', cpu_shares=1,
                                       memory='256M', privileged=True,
                                       target_num_containers=3, run_command='-d',
                                       entrypoint='/bin/mysql', container_ports=ports,
                                       container_envvars=utils.parse_envvars(container_envvars, []),
                                       linked_to_service=utils.parse_links(linked_to_service, 'to_service'),
                                       autorestart='OFF', autodestroy='OFF', autoredeploy='OFF',
                                       roles='poweruser', sequential_deployment=True, tags=[], bindings=[],
                                       deployment_strategy=None)
        mock_save.assert_called()
        mock_start.assert_called()
        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Service.create', side_effect=TutumApiError)
    def test_service_run_with_exception(self, mock_create, mock_exit):
        exposed_ports = ['80', '22']
        published_ports = ['80:80/tcp', '22:22']
        container_envvars = ['MYSQL_ADMIN=admin', 'MYSQL_PASS=password']
        linked_to_service = ['mysql:mysql', 'redis:redis']
        service_run('imagename', 'containername', 1, '256M', True, 3, '-d', '/bin/mysql',
                    exposed_ports, published_ports, container_envvars, [], '', linked_to_service,
                    'OFF', 'OFF', 'OFF', 'poweruser', True, None, None, None)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceScaleTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.save')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service')
    def test_service_scale(self, mock_fetch_remote_service, mock_save):
        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_service.return_value = service
        service_scale(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'], 3)

        mock_save.assert_called()
        self.assertEqual(3, service.target_num_containers)
        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service', side_effect=TutumApiError)
    def test_service_scale_with_exception(self, mock_fetch_remote_service, mock_exit):
        service_scale(['test_id'], 3)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceSetTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.save')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service')
    def test_service_set(self, mock_fetch_remote_service, mock_save):
        service = tutumcli.commands.tutum.Service()
        exposed_ports = [80]
        published_ports = ['800:80/tcp']
        ports = [{'inner_port': '80', 'outer_port': '800', 'protocol': 'tcp', 'published': True}]
        container_envvars = ['MYSQL_ADMIN=admin', 'MYSQL_PASS=password']
        linked_to_service = ['mysql:mysql', 'redis:redis']
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'

        mock_fetch_remote_service.return_value = service
        service_set([service.uuid], 'imagename', 1, '256M', True, 3, '-d', '/bin/mysql',
                    exposed_ports, published_ports, container_envvars, [], '', linked_to_service,
                    'OFF', 'OFF', 'OFF', 'poweruser', True, False, None, None, None)

        mock_save.assert_called()
        self.assertEqual('7A4CFE51-03BB-42D6-825E-3B533888D8CD\n'
                         'Service must be redeployed to have its configuration changes applied.\n'
                         'To redeploy execute: $ tutum service redeploy 7A4CFE51-03BB-42D6-825E-3B533888D8CD',
                         self.buf.getvalue().strip())
        self.assertEqual(1, service.cpu_shares)
        self.assertEqual('256M', service.memory)
        self.assertEqual(True, service.privileged)
        self.assertEqual(3, service.target_num_containers)
        self.assertEqual('-d', service.run_command)
        self.assertEqual('/bin/mysql', service.entrypoint)
        self.assertEqual(ports, service.container_ports)
        self.assertEqual(utils.parse_envvars(container_envvars, []), service.container_envvars)
        self.assertEqual(utils.parse_links(linked_to_service, 'to_service'), service.linked_to_service)
        self.assertEqual('OFF', service.autorestart)
        self.assertEqual('OFF', service.autodestroy)
        self.assertEqual('OFF', service.autoredeploy)
        self.assertEqual('poweruser', service.roles)
        self.assertEqual(True, service.sequential_deployment)
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service', side_effect=TutumApiError)
    def test_service_set_with_exception(self, mock_fetch_remote_service, mock_exit):
        service = tutumcli.commands.tutum.Service()
        exposed_ports = [80]
        published_ports = ['800:80/tcp']
        container_envvars = ['MYSQL_ADMIN=admin', 'MYSQL_PASS=password']
        linked_to_service = ['mysql:mysql', 'redis:redis']

        mock_fetch_remote_service.return_value = service
        service_set(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'], 'imagename', 1, '256M', True, 3, '-d', '/bin/mysql',
                    exposed_ports, published_ports, container_envvars, [], '', linked_to_service,
                    'OFF', 'OFF', 'OFF', 'poweruser', True, False, None, None, None)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceStartTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.start')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service')
    def test_service_start(self, mock_fetch_remote_service, mock_start):
        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_service.return_value = service
        mock_start.return_value = True
        service_start(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service', side_effect=TutumApiError)
    def test_service_start_with_exception(self, mock_fetch_remote_service, mock_exit):
        service_start(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceStopTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.stop')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service')
    def test_service_stop(self, mock_fetch_remote_service, mock_stop):
        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_service.return_value = service
        mock_stop.return_value = True
        service_stop(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service', side_effect=TutumApiError)
    def test_service_stop_with_exception(self, mock_fetch_remote_service, mock_exit):
        service_start(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceTerminateTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.delete')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service')
    def test_service_teminate(self, mock_fetch_remote_service, mock_delete):
        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_service.return_value = service
        mock_delete.return_value = True
        service_terminate(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service', side_effect=TutumApiError)
    def test_service_terminate_with_exception(self, mock_fetch_remote_service, mock_exit):
        service_terminate(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ServiceRedeployTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Service.redeploy')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service')
    def test_service_teminate(self, mock_fetch_remote_service, mock_redeploy):
        service = tutumcli.commands.tutum.Service()
        service.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_service.return_value = service
        mock_redeploy.return_value = True
        service_redeploy(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(service.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_service', side_effect=TutumApiError)
    def test_service_terminate_with_exception(self, mock_fetch_remote_service, mock_exit):
        service_redeploy(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ContainerInspectTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Container.get_all_attributes')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container')
    def test_container_inspect(self, mock_fetch_remote_container, mock_get_all_attributes):
        output = '''{
  "key": [
    {
      "name": "test",
      "id": "1"
    }
  ]
}'''
        uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        container = tutumcli.commands.tutum.Container()
        container.uuid = uuid
        mock_fetch_remote_container.return_value = container
        mock_get_all_attributes.return_value = {'key': [{'name': 'test', 'id': '1'}]}
        container_inspect(['test_id'])

        self.assertEqual(' '.join(output.split()), ' '.join(self.buf.getvalue().strip().split()))
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container', side_effect=TutumApiError)
    def test_container_inspect_with_exception(self, mock_fetch_remote_container, mock_exit):
        container = tutumcli.commands.tutum.Container()
        mock_fetch_remote_container.return_value = container
        container_inspect(['test_id', 'test_id2'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ContainerLogsTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.utils.fetch_remote_container')
    def test_container_logs(self, mock_fetch_remote_container):
        log = 'Here is the log'
        container = tutumcli.commands.tutum.Container
        container.logs = log
        mock_fetch_remote_container.return_value = container
        container_logs(['test_id'])

        self.assertEqual(log, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container', side_effect=TutumApiError)
    def test_container_logs_with_exception(self, mock_fetch_remote_container, mock_exit):
        container = tutumcli.commands.tutum.Container()
        mock_fetch_remote_container.return_value = container
        container_logs(['test_id', 'test_id2'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ContainerPsTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

        container1 = tutumcli.commands.tutum.Container()
        container1.name = 'CONTAINER1'
        container1.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        container1.image_name = 'test/container1'
        container1.public_dns = 'container1.io'
        container1.state = 'Running'
        container1.deployed_datetime = ''
        container1.run_command = '/bin/bash'
        container1.container_ports = [{'protocol': 'tcp', 'inner_port': 8080, 'outer_port': 8080}]
        container1.exit_code = 1
        container1.service = 'container1_service_uri'
        container2 = tutumcli.commands.tutum.Container()
        container2.name = 'CONTAINER2'
        container2.uuid = '8B4CFE51-03BB-42D6-825E-3B533888D8CD'
        container2.image_name = 'test/container2'
        container2.public_dns = 'container2.io'
        container2.state = 'Stopped'
        container2.deployed_datetime = ''
        container2.run_command = '/bin/sh'
        container2.container_ports = [{'protocol': 'tcp', 'inner_port': 3306, 'outer_port': 3307}]
        container2.exit_code = 0
        container2.service = 'container2_service_uri'
        self.containerlist = [container1, container2]
        service1 = tutumcli.commands.tutum.Service()
        service1.resource_uri = 'container1_service_uri'
        service1.stack = 'container1_service_stack_uri'
        service2 = tutumcli.commands.tutum.Service()
        service2.resource_uri = 'container2_service_uri'
        service2.stack = 'container2_service_stack_uri'
        self.servicelist = [service1, service2]
        stack1 = tutumcli.commands.tutum.Stack()
        stack1.resource_uri = 'container1_service_stack_uri'
        stack1.name = 'service1'
        stack2 = tutumcli.commands.tutum.Stack()
        stack2.resource_uri = 'container2_service_stack_uri'
        stack2.name = 'service2'
        self.stacklist = [stack1, stack2]

    def tearDown(self):
        pass
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Stack.list')
    @mock.patch('tutumcli.commands.tutum.Service.list')
    @mock.patch('tutumcli.commands.tutum.Container.list')
    def test_container_ps(self, mock_list, mock_service, mock_stack):
        output = u'''NAME        UUID      STATUS     IMAGE            RUN COMMAND      EXIT CODE  DEPLOYED    PORTS                         STACK
CONTAINER1  7A4CFE51  ▶ Running  test/container1  /bin/bash                1              container1.io:8080->8080/tcp  service1
CONTAINER2  8B4CFE51  ◼ Stopped  test/container2  /bin/sh                  0              container2.io:3307->3306/tcp  service2'''
        mock_stack.return_value = self.stacklist
        mock_service.return_value = self.servicelist
        mock_list.return_value = self.containerlist
        container_ps(False, 'Running', None)

        mock_list.assert_called_with(state='Running', service=None)
        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.Stack.list')
    @mock.patch('tutumcli.commands.tutum.Service.list')
    @mock.patch('tutumcli.commands.tutum.Container.list')
    def test_container_ps_quiet(self, mock_list, mock_service, mock_stack):
        output = '''7A4CFE51-03BB-42D6-825E-3B533888D8CD
8B4CFE51-03BB-42D6-825E-3B533888D8CD'''
        mock_stack.return_value = self.stacklist
        mock_service.return_value = self.servicelist
        mock_list.return_value = self.containerlist
        container_ps(True, None, None)
        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)


    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Container.list', side_effect=TutumApiError)
    def test_container_ps_with_exception(self, mock_list, mock_exit):
        container_ps(None, None, None)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ContainerStartTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Container.start')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container')
    def test_container_start(self, mock_fetch_remote_container, mock_start):
        container = tutumcli.commands.tutum.Container()
        container.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_container.return_value = container
        mock_start.return_value = True
        container_start(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(container.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container', side_effect=TutumApiError)
    def test_container_start_with_exception(self, mock_fetch_remote_container, mock_exit):
        container_start(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ContainerStopTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Container.stop')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container')
    def test_container_stop(self, mock_fetch_remote_container, mock_stop):
        container = tutumcli.commands.tutum.Container()
        container.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_container.return_value = container
        mock_stop.return_value = True
        container_stop(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(container.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container', side_effect=TutumApiError)
    def test_container_stop_with_exception(self, mock_fetch_remote_container, mock_exit):
        container_start(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ContainerTerminateTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Container.delete')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container')
    def test_container_teminate(self, mock_fetch_remote_container, mock_delete):
        container = tutumcli.commands.tutum.Container()
        container.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_container.return_value = container
        mock_delete.return_value = True
        container_terminate(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(container.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_container', side_effect=TutumApiError)
    def test_container_terminate_with_exception(self, mock_fetch_remote_container, mock_exit):
        container_terminate(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ImageListTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

        image1 = tutumcli.commands.tutum.Image()
        image1.name = 'image_name_1'
        image1.description = 'image_desc_1'
        image2 = tutumcli.commands.tutum.Image()
        image2.name = 'image_name_2'
        image2.description = 'image_desc_2'
        self.imagelist = [image1, image2]

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Image.list')
    def test_image_list(self, mock_list):
        output = u'''NAME          DESCRIPTION
image_name_1  image_desc_1
image_name_2  image_desc_2'''
        mock_list.return_value = self.imagelist
        image_list(False, False, False)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.Image.list')
    def test_image_list_quiet(self, mock_list):
        output = 'image_name_1\nimage_name_2'
        mock_list.return_value = self.imagelist
        image_list(True, False, False)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.Image.list', return_value=[])
    def test_image_list_parameter(self, mock_list):
        image_list(False, True, False)
        mock_list.assert_called_with(starred=True)

        image_list(False, False, True)
        mock_list.assert_called_with(base_image=True)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Image.list', side_effect=TutumApiError)
    def test_image_list_with_exception(self, mock_fetch_remote_container, mock_exit):
        image_list(False, False, False)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ImageRegister(unittest.TestCase):
    def setUp(self):
        self.raw_input_holder = __builtin__.raw_input
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()


    def tearDown(self):
        sys.stdout = self.stdout
        __builtin__.raw_input = self.raw_input_holder

    @mock.patch('tutumcli.commands.tutum.Image.save', return_value=True)
    @mock.patch('tutumcli.commands.tutum.Image.create')
    @mock.patch('tutumcli.commands.getpass.getpass', return_value='password')
    def test_register(self, mock_get_pass, mock_create, mock_save):
        output = '''Please input username and password of the registry:
image_name'''
        __builtin__.raw_input = lambda _: 'username'  # set username
        image = tutumcli.commands.tutum.Image()
        image.name = 'image_name'
        mock_create.return_value = image
        image_register('repository', 'descripiton', None, None)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Image.create', side_effect=TutumApiError)
    @mock.patch('tutumcli.commands.getpass.getpass', return_value='password')
    def test_register_with_exception(self, mock_get_pass, mock_create, mock_exit):
        __builtin__.raw_input = lambda _: 'username'  # set username
        image_register('repository', 'descripiton', None, None)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ImageRmTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Image.delete', return_value=True)
    @mock.patch('tutumcli.commands.tutum.Image.fetch')
    def test_image_rm(self, mock_fetch, mock_delete):
        image_rm(['repo1', 'repo2'])

        self.assertEqual('repo1\nrepo2', self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Image.delete', return_value=True)
    @mock.patch('tutumcli.commands.tutum.Image.fetch')
    def test_image_rm_with_exception(self, mock_fetch, mock_delete, mock_exit):
        mock_fetch.side_effect = [tutumcli.commands.tutum.Image(), TutumApiError]
        image_rm(['repo1', 'repo2'])

        self.assertEqual('repo1', self.buf.getvalue().strip())
        self.buf.truncate(0)
        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ImageSearchTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch.object(tutumcli.commands.utils.docker.Client, 'search')
    @mock.patch('tutumcli.utils.get_docker_client')
    def test_image_search(self, mock_get_docker_client, mock_search):
        mock_get_docker_client.return_value = docker.Client()
        mock_search.return_value = [
            {
                "description": "1st image",
                "is_official": True,
                "is_trusted": False,
                "name": "wma55/u1210sshd",
                "star_count": 0
            },
            {
                "description": "2nd image",
                "is_official": False,
                "is_trusted": True,
                "name": "jdswinbank/sshd",
                "star_count": 0
            },
            {
                "description": "3rd image",
                "is_official": True,
                "is_trusted": True,
                "name": "vgauthier/sshd",
                "star_count": 0
            }]
        output = u'''NAME             DESCRIPTION      STARS  OFFICIAL    TRUSTED
wma55/u1210sshd  1st image            0  ✓
jdswinbank/sshd  2nd image            0              ✓
vgauthier/sshd   3rd image            0  ✓           ✓'''
        image_search('keyword')
        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch.object(tutumcli.commands.utils.docker.Client, 'search', side_effect=TutumApiError)
    @mock.patch('tutumcli.utils.get_docker_client')
    def test_image_search_with_exception(self, mock_get_docker_client, mock_search, mock_exit):
        mock_get_docker_client.return_value = docker.Client()
        image_search('keyword')

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class ImageUpdateTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Image.save', return_value=True)
    @mock.patch('tutumcli.commands.tutum.Image.fetch')
    def test_image_update(self, mock_fetch, mock_save):
        image = tutumcli.commands.tutum.Image()
        image.name = 'name'
        mock_fetch.return_value = image
        mock_save.return_value = True
        image_update(['repo'], 'username', 'password', 'description')
        self.assertEqual('username', image.username)
        self.assertEqual('password', image.password)
        self.assertEqual('description', image.description)
        self.assertEqual('name', self.buf.getvalue().strip())

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Image.fetch')
    def test_image_update_with_exception(self, mock_fetch, mock_exit):
        mock_fetch.side_effect = TutumApiError
        image_update(['repo'], 'username', 'password', 'description')

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeListTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()
        node1 = tutumcli.commands.tutum.Node()
        node1.uuid = '19303d01-3564-437b-ac54-e7f8d17003f6'
        node1.external_fqdn = '19303d01-tifayuki.node.tutum.io'
        node1.state = 'Deployed'
        node1.last_seen = None
        node1.node_cluster = '/api/v1/nodecluster/b0374cc2-4003-4270-b131-25fc494ea2be/'
        node2 = tutumcli.commands.tutum.Node()
        node2.uuid = 'bd276db4-cd35-4311-8110-1c82885c33d2'
        node2.external_fqdn = 'bd276db4-tifayuki.node.tutum.io"'
        node2.state = 'Deploying'
        node2.last_seen = None
        node2.node_cluster = '/api/v1/nodecluster/b0374cc2-4003-4270-b131-25fc494ea2be/'
        self.nodeklist = [node1, node2]

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.NodeCluster.fetch')
    @mock.patch('tutumcli.commands.tutum.Node.list')
    def test_node_list(self, mock_list, mock_fetch):
        output = u'''UUID      FQDN                              LASTSEEN    STATUS       CLUSTER
19303d01  19303d01-tifayuki.node.tutum.io               ▶ Deployed   test_nodecluster
bd276db4  bd276db4-tifayuki.node.tutum.io"              ⚙ Deploying  test_nodecluster'''
        mock_list.return_value = self.nodeklist
        nodecluster = tutumcli.commands.tutum.NodeCluster()
        nodecluster.name = 'test_nodecluster'
        mock_fetch.return_value = nodecluster
        node_list(quiet=False)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.NodeCluster.fetch')
    @mock.patch('tutumcli.commands.tutum.Node.list')
    def test_node_list(self, mock_list, mock_fetch):
        output = '''19303d01-3564-437b-ac54-e7f8d17003f6
bd276db4-cd35-4311-8110-1c82885c33d2'''
        mock_list.return_value = self.nodeklist
        nodecluster = tutumcli.commands.tutum.NodeCluster()
        nodecluster.name = 'test_nodecluster'
        mock_fetch.return_value = nodecluster
        node_list(quiet=True)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Node.list', side_effect=TutumApiError)
    def test_node_list(self, mock_list, mock_exit):
        node_list(False)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeInspectTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Node.get_all_attributes')
    @mock.patch('tutumcli.commands.tutum.Node.fetch')
    @mock.patch('tutumcli.commands.utils.fetch_remote_node')
    def test_node_inspect(self, mock_fetch_remote_node, mock_fetch, mock_get_all_attributes):
        output = '''{
  "key": [
    {
      "name": "test",
      "id": "1"
    }
  ]
}'''
        uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        node = tutumcli.commands.tutum.Node()
        node.uuid = uuid
        mock_fetch.return_value = node
        mock_fetch_remote_node.return_value = node
        mock_get_all_attributes.return_value = {'key': [{'name': 'test', 'id': '1'}]}
        node_inspect(['test_id'])

        mock_fetch.assert_called_with(uuid)
        self.assertEqual(' '.join(output.split()), ' '.join(self.buf.getvalue().strip().split()))
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_node', side_effect=TutumApiError)
    def test_node_inspect_with_exception(self, mock_fetch_remote_node, mock_exit):
        node = tutumcli.commands.tutum.Node()
        mock_fetch_remote_node.return_value = node
        node_inspect(['test_id', 'test_id2'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeRmTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Node.delete')
    @mock.patch('tutumcli.commands.utils.fetch_remote_node')
    def test_node_teminate(self, mock_fetch_remote_node, mock_delete):
        node = tutumcli.commands.tutum.Node()
        node.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_node.return_value = node
        mock_delete.return_value = True
        node_rm(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(node.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_node', side_effect=TutumApiError)
    def test_node_terminate_with_exception(self, mock_fetch_remote_node, mock_exit):
        node_rm(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeClusterListTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()
        nodecluster1 = tutumcli.commands.tutum.NodeCluster()
        nodecluster1.name = 'test_sfo'
        nodecluster1.uuid = 'b0374cc2-4003-4270-b131-25fc494ea2be'
        nodecluster1.region = '/api/v1/region/digitalocean/sfo1/'
        nodecluster1.node_type = '/api/v1/nodetype/digitalocean/512mb/'
        nodecluster1.deployed_datetime = None
        nodecluster1.state = 'Deployed'
        nodecluster1.current_num_nodes = 2
        nodecluster1.target_num_nodes = 2
        nodecluster2 = tutumcli.commands.tutum.NodeCluster()
        nodecluster2.name = 'newyork3'
        nodecluster2.uuid = 'a4c1e712-ca26-4547-adb7-8da1057b964b'
        nodecluster2.region = '/api/v1/region/digitalocean/nyc3/'
        nodecluster2.node_type = '/api/v1/nodetype/digitalocean/512mb/'
        nodecluster2.deployed_datetime = None
        nodecluster2.state = 'Provisioning'
        nodecluster2.current_num_nodes = 1
        nodecluster2.target_num_nodes = 1
        self.nodeclusterlist = [nodecluster1, nodecluster2]

        region1 = tutumcli.commands.tutum.Region()
        region1.label = 'New York 3'
        region2 = tutumcli.commands.tutum.Region()
        region2.label = 'San Francisco 1'
        self.regionlist = [region1, region2]

        nodetype1 = tutumcli.commands.tutum.NodeType()
        nodetype1.label = '512MB'
        nodetype2 = tutumcli.commands.tutum.NodeType()
        nodetype2.label = '512MB'
        self.nodetypelist = [nodetype1, nodetype2]


    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Region.fetch')
    @mock.patch('tutumcli.commands.tutum.NodeType.fetch')
    @mock.patch('tutumcli.commands.tutum.NodeCluster.list')
    def test_clusternode_list(self, mock_list, mock_nodetype_fetch, mock_region_fetch):
        mock_list.return_value = self.nodeclusterlist
        mock_nodetype_fetch.side_effect = self.nodetypelist
        mock_region_fetch.side_effect = self.regionlist
        output = '''NAME      UUID      REGION           TYPE    DEPLOYED    STATUS          CURRENT#NODES    TARGET#NODES
test_sfo  b0374cc2  New York 3       512MB               Deployed                    2               2
newyork3  a4c1e712  San Francisco 1  512MB               Provisioning                1               1'''
        nodecluster_list(quiet=False)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)


    @mock.patch('tutumcli.commands.tutum.Region.fetch')
    @mock.patch('tutumcli.commands.tutum.NodeType.fetch')
    @mock.patch('tutumcli.commands.tutum.NodeCluster.list')
    def test_clusternode_list_quiet(self, mock_list, mock_nodetype_fetch, mock_region_fetch):
        mock_list.return_value = self.nodeclusterlist
        mock_nodetype_fetch.side_effect = self.nodetypelist
        mock_region_fetch.side_effect = self.regionlist
        output = 'b0374cc2-4003-4270-b131-25fc494ea2be\na4c1e712-ca26-4547-adb7-8da1057b964b'
        nodecluster_list(quiet=True)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.NodeCluster.list', side_effect=TutumApiError)
    def test_clusternode_list_with_excepiton(self, mock_list, mock_exit):
        nodecluster_list(quiet=True)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeClusterInspectTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.NodeCluster.get_all_attributes')
    @mock.patch('tutumcli.commands.tutum.NodeCluster.fetch')
    @mock.patch('tutumcli.commands.utils.fetch_remote_nodecluster')
    def test_nodecluster_inspect(self, mock_fetch_remote_node_cluster, mock_fetch, mock_get_all_attributes):
        output = '''{
  "key": [
    {
      "name": "test",
      "id": "1"
    }
  ]
}'''
        uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        nodecluster = tutumcli.commands.tutum.NodeCluster()
        nodecluster.uuid = uuid
        mock_fetch.return_value = nodecluster
        mock_fetch_remote_node_cluster.return_value = nodecluster
        mock_get_all_attributes.return_value = {'key': [{'name': 'test', 'id': '1'}]}
        nodecluster_inspect(['test_id'])

        mock_fetch.assert_called_with(uuid)
        self.assertEqual(' '.join(output.split()), ' '.join(self.buf.getvalue().strip().split()))
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_nodecluster', side_effect=TutumApiError)
    def test_nodecluster_inspect_with_exception(self, mock_fetch_remote_nodecluster, mock_exit):
        nodecluster = tutumcli.commands.tutum.NodeCluster()
        mock_fetch_remote_nodecluster.return_value = nodecluster
        nodecluster_inspect(['test_id', 'test_id2'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeClusterRmTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.NodeCluster.delete')
    @mock.patch('tutumcli.commands.utils.fetch_remote_nodecluster')
    def test_nodecluster_rm(self, mock_fetch_remote_nodecluster, mock_delete):
        nodecluster = tutumcli.commands.tutum.NodeCluster()
        nodecluster.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_nodecluster.return_value = nodecluster
        mock_delete.return_value = True
        nodecluster_rm(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        self.assertEqual(nodecluster.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_nodecluster', side_effect=TutumApiError)
    def test_nodecluster_rm_with_exception(self, mock_fetch_remote_nodecluster, mock_exit):
        nodecluster_rm(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'])

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeClusterScaleTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.NodeCluster.save')
    @mock.patch('tutumcli.commands.utils.fetch_remote_nodecluster')
    def test_nodecluster_scale(self, mock_fetch_remote_nodecluster, mock_save):
        nodecluster = tutumcli.commands.tutum.NodeCluster()
        nodecluster.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_fetch_remote_nodecluster.return_value = nodecluster
        nodecluster_scale(['7A4CFE51-03BB-42D6-825E-3B533888D8CD'], 3)

        mock_save.assert_called()
        self.assertEqual(3, nodecluster.target_num_nodes)
        self.assertEqual(nodecluster.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.utils.fetch_remote_nodecluster', side_effect=TutumApiError)
    def test_nodecluster_scale_with_exception(self, mock_fetch_remote_nodecluster, mock_exit):
        nodecluster_scale(['test_id'], 3)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeClusterShowProviderTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

        provider = tutumcli.commands.tutum.Provider()
        provider.name = 'digitalocean'
        provider.label = 'Digital Ocean'
        provider.regions = [
            "/api/v1/region/digitalocean/ams1/",
            "/api/v1/region/digitalocean/ams2/",
            "/api/v1/region/digitalocean/ams3/",
            "/api/v1/region/digitalocean/lon1/",
            "/api/v1/region/digitalocean/nyc1/",
            "/api/v1/region/digitalocean/nyc2/",
            "/api/v1/region/digitalocean/nyc3/",
            "/api/v1/region/digitalocean/sfo1/",
            "/api/v1/region/digitalocean/sgp1/"
        ]
        self.providerlist = [provider]

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Provider.list')
    def test_nodecluster_show_providers(self, mock_list):
        output = '''NAME          LABEL          REGIONS
digitalocean  Digital Ocean  ams1, ams2, ams3, lon1, nyc1, nyc2, nyc3, sfo1, sgp1'''
        mock_list.return_value = self.providerlist
        nodecluster_show_providers(quiet=False)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.Provider.list')
    def test_nodecluster_show_providers_quiet(self, mock_list):
        output = 'digitalocean'
        mock_list.return_value = self.providerlist
        nodecluster_show_providers(quiet=True)

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Provider.list', side_effect=TutumApiError)
    def test_nodecluster_show_providers_with_exception(self, mock_list, mock_exit):
        nodecluster_show_providers(quiet=True)

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeClusterShowRegionsTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()
        region1 = tutumcli.commands.tutum.Region()
        region1.name = 'ams1'
        region1.label = 'Amsterdam 1'
        region1.resource_uri = '/api/v1/region/digitalocean/ams1/'
        region1.node_types = ["/api/v1/nodetype/digitalocean/512mb/",
                              "/api/v1/nodetype/digitalocean/1gb/",
                              "/api/v1/nodetype/digitalocean/2gb/",
                              "/api/v1/nodetype/digitalocean/4gb/",
                              "/api/v1/nodetype/digitalocean/8gb/",
                              "/api/v1/nodetype/digitalocean/16gb/"]
        region2 = tutumcli.commands.tutum.Region()
        region2.name = 'sfo1'
        region2.label = 'San Francisco 1'
        region2.resource_uri = '/api/v1/region/digitalocean/sfo1/'
        region2.node_types = ["/api/v1/nodetype/digitalocean/512mb/",
                              "/api/v1/nodetype/digitalocean/1gb/",
                              "/api/v1/nodetype/digitalocean/2gb/",
                              "/api/v1/nodetype/digitalocean/4gb/",
                              "/api/v1/nodetype/digitalocean/8gb/",
                              "/api/v1/nodetype/digitalocean/16gb/",
                              "/api/v1/nodetype/digitalocean/32gb/",
                              "/api/v1/nodetype/digitalocean/48gb/",
                              "/api/v1/nodetype/digitalocean/64gb/"]
        region3 = tutumcli.commands.tutum.Region()
        region3.name = 'jap1'
        region3.label = 'Japan 1'
        region3.resource_uri = '/api/v1/region/aws/jap1/'
        region3.node_types = ["/api/v1/nodetype/aws/512mb/",
                              "/api/v1/nodetype/aws/1gb/",
                              "/api/v1/nodetype/aws/2gb/",
                              "/api/v1/nodetype/aws/4gb/",
                              "/api/v1/nodetype/aws/8gb/"]
        self.regionlist = [region1, region2, region3]

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.Region.list')
    def test_nodecluster_show_regions(self, mock_list):
        output = '''NAME    LABEL            PROVIDER      TYPE
ams1    Amsterdam 1      digitalocean  512mb, 1gb, 2gb, 4gb, 8gb, 16gb
sfo1    San Francisco 1  digitalocean  512mb, 1gb, 2gb, 4gb, 8gb, 16gb, 32gb, 48gb, 64gb
jap1    Japan 1          aws           512mb, 1gb, 2gb, 4gb, 8gb'''
        mock_list.return_value = self.regionlist
        nodecluster_show_regions('')

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.Region.list')
    def test_nodecluster_show_regions_with_filter(self, mock_list):
        output = '''NAME    LABEL            PROVIDER      TYPE
ams1    Amsterdam 1      digitalocean  512mb, 1gb, 2gb, 4gb, 8gb, 16gb
sfo1    San Francisco 1  digitalocean  512mb, 1gb, 2gb, 4gb, 8gb, 16gb, 32gb, 48gb, 64gb'''
        mock_list.return_value = self.regionlist
        nodecluster_show_regions('digitalocean')

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.Region.list', side_effect=TutumApiError)
    def test_nodecluster_show_regions_with_exception(self, mock_list, mock_exit):
        nodecluster_show_regions('')

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeClusterShowTypesTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()
        nodetype1 = tutumcli.commands.tutum.NodeType()
        nodetype1.name = '512mb'
        nodetype1.label = '512MB'
        nodetype1.resource_uri = '/api/v1/nodetype/digitalocean/512mb/'
        nodetype1.regions = ["/api/v1/region/digitalocean/ams1/",
                             "/api/v1/region/digitalocean/sfo1/",
                             "/api/v1/region/digitalocean/nyc2/",
                             "/api/v1/region/digitalocean/ams2/",
                             "/api/v1/region/digitalocean/sgp1/",
                             "/api/v1/region/digitalocean/lon1/",
                             "/api/v1/region/digitalocean/nyc3/",
                             "/api/v1/region/digitalocean/nyc1/"]
        nodetype2 = tutumcli.commands.tutum.NodeType()
        nodetype2.name = '1gb'
        nodetype2.label = '1GB'
        nodetype2.resource_uri = '/api/v1/nodetype/digitalocean/1gb/'
        nodetype2.regions = ["/api/v1/region/digitalocean/ams1/",
                             "/api/v1/region/digitalocean/sfo1/",
                             "/api/v1/region/digitalocean/nyc2/",
                             "/api/v1/region/digitalocean/ams2/",
                             "/api/v1/region/digitalocean/sgp1/",
                             "/api/v1/region/digitalocean/lon1/",
                             "/api/v1/region/digitalocean/nyc3/",
                             "/api/v1/region/digitalocean/nyc1/"]
        nodetype3 = tutumcli.commands.tutum.NodeType()
        nodetype3.name = '3gb'
        nodetype3.label = '3GB'
        nodetype3.resource_uri = '/api/v1/region/aws/3gb/'
        nodetype3.regions = ["/api/v1/nodetype/aws/tokyo/",
                             "/api/v1/nodetype/aws/kyoto/",
                             "/api/v1/nodetype/aws/shibuya/",
                             "/api/v1/nodetype/aws/ueno/",
                             "/api/v1/nodetype/aws/akiba/"]
        self.nodetypelist = [nodetype1, nodetype2, nodetype3]

    def tearDown(self):
        sys.stdout = self.stdout

    @mock.patch('tutumcli.commands.tutum.NodeType.list')
    def test_nodecluster_show_types(self, mock_list):
        output = '''NAME    LABEL    PROVIDER      REGIONS
512mb   512MB    digitalocean  ams1, sfo1, nyc2, ams2, sgp1, lon1, nyc3, nyc1
1gb     1GB      digitalocean  ams1, sfo1, nyc2, ams2, sgp1, lon1, nyc3, nyc1
3gb     3GB      aws           tokyo, kyoto, shibuya, ueno, akiba'''
        mock_list.return_value = self.nodetypelist
        nodecluster_show_types('', '')

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.NodeType.list')
    def test_nodecluster_show_types_with_provider_filter(self, mock_list):
        output = '''NAME    LABEL    PROVIDER    REGIONS
3gb     3GB      aws         tokyo, kyoto, shibuya, ueno, akiba'''
        mock_list.return_value = self.nodetypelist
        nodecluster_show_types('aws', '')

        self.assertEqual(output, self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.tutum.NodeType.list')
    def test_nodecluster_show_types_with_region_filter(self, mock_list):
        output = '''NAME    LABEL    PROVIDER      REGIONS
512mb   512MB    digitalocean  ams1, sfo1, nyc2, ams2, sgp1, lon1, nyc3, nyc1
1gb     1GB      digitalocean  ams1, sfo1, nyc2, ams2, sgp1, lon1, nyc3, nyc1'''
        mock_list.return_value = self.nodetypelist
        nodecluster_show_types(output, 'nyc3')

    @mock.patch('tutumcli.commands.tutum.NodeType.list')
    def test_nodecluster_show_types_with_filters(self, mock_list):
        mock_list.return_value = self.nodetypelist
        nodecluster_show_types('aws', 'nyc3')

        self.assertEqual('NAME    LABEL    PROVIDER    REGIONS', self.buf.getvalue().strip())
        self.buf.truncate(0)

    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.NodeType.list', side_effect=TutumApiError)
    def test_nodecluster_show_types_with_exception(self, mock_list, mock_exit):
        nodecluster_show_types('', '')

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)


class NodeClusterCreateTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout


    @mock.patch('tutumcli.commands.tutum.NodeCluster.deploy')
    @mock.patch('tutumcli.commands.tutum.NodeCluster.save')
    @mock.patch('tutumcli.commands.tutum.NodeCluster.create')
    def test_nodecluster_create(self, mock_create, mock_save, mock_deploy):
        provider_name = 'digitalocean'
        region_name = 'lon1'
        nodetype_name = '1gb'
        region_uri = "/api/v1/region/%s/%s/" % (provider_name, region_name)
        nodetype_uri = "/api/v1/nodetype/%s/%s/" % (provider_name, nodetype_name)
        nodecluster = tutumcli.commands.tutum.NodeCluster()
        nodecluster.uuid = '7A4CFE51-03BB-42D6-825E-3B533888D8CD'
        mock_create.return_value = nodecluster
        nodecluster_create(3, 'name', provider_name, region_name, nodetype_name)

        mock_create.assert_called_with(name='name', target_num_nodes=3, region=region_uri,
                                       node_type=nodetype_uri)
        self.assertEqual(nodecluster.uuid, self.buf.getvalue().strip())
        self.buf.truncate(0)


    @mock.patch('tutumcli.commands.sys.exit')
    @mock.patch('tutumcli.commands.tutum.NodeCluster.create', side_effect=TutumApiError)
    def test_nodecluster_create_with_exception(self, mock_create, mock_exit):
        nodecluster_create(3, 'name', 'provider', 'region', 'nodetype')

        mock_exit.assert_called_with(EXCEPTION_EXIT_CODE)



import argparse
import logging
import copy
import sys
import codecs

import requests

from . import __version__
from tutumcli import parsers
from tutumcli import commands
from tutumcli.exceptions import InternalError


requests.packages.urllib3.disable_warnings()

sys.stdout = codecs.getwriter('utf8')(sys.stdout)

logging.basicConfig()


def initialize_parser():
    # Top parser
    parser = argparse.ArgumentParser(description="Tutum's CLI", prog='tutum')
    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument('--debug', action='store_true', help=argparse.SUPPRESS)
    subparsers = parser.add_subparsers(title="Tutum's CLI commands", dest='cmd')
    # Command Parsers
    parsers.add_build_parser(subparsers)
    parsers.add_container_parser(subparsers)
    parsers.add_image_parser(subparsers)
    parsers.add_login_parser(subparsers)
    parsers.add_node_parser(subparsers)
    parsers.add_nodecluster_parser(subparsers)
    parsers.add_service_parser(subparsers)
    parsers.add_stack_parser(subparsers)
    parsers.add_tag_parser(subparsers)
    parsers.add_volume_parser(subparsers)
    parsers.add_volumegroup_parser(subparsers)
    parsers.add_webhookhandler_parser(subparsers)
    return parser


def patch_help_option(argv=sys.argv):
    args = copy.copy(argv)

    if not args:
        raise InternalError("Wrong argument is set, cannot be empty")
    debug = False
    if len(args) >= 2 and args[1] == '--debug':
        debug = True
        args.pop(1)

    if len(args) == 1:
        args.append('-h')
    elif len(args) == 2 and args[1] in ['service', 'build', 'container', 'image', 'node', 'nodecluster', 'tag',
                                        'volume', 'volumegroup', 'webhook-handler', 'stack']:
        args.append('-h')
    elif len(args) == 3:
        if args[1] == 'service' and args[2] in ['create', 'inspect', 'logs', 'redeploy', 'run', 'scale', 'set',
                                                'start', 'stop', 'terminate']:
            args.append('-h')
        elif args[1] == 'container' and args[2] in ['inspect', 'logs', 'redeploy', 'start', 'stop',
                                                    'terminate']:
            args.append('-h')
        elif args[1] == 'image' and args[2] in ['register', 'push', 'rm', 'search', 'update']:
            args.append('-h')
        elif args[1] == 'node' and args[2] in ['inspect', 'rm', 'upgrade']:
            args.append('-h')
        elif args[1] == 'nodecluster' and args[2] in ['create', 'inspect', 'rm', 'scale']:
            args.append('-h')
        elif args[1] == 'tag' and args[2] in ['add', 'list', 'rm', 'set']:
            args.append('-h')
        elif args[1] == 'webhook-handler' and args[2] in ['create', 'list', 'rm']:
            args.append('-h')
        elif args[1] == 'volume' and args[2] in ['inspect']:
            args.append('-h')
        elif args[1] == 'volumegroup' and args[2] in ['inspect']:
            args.append('-h')
        elif args[1] == 'stack' and args[2] in ['inspect', 'redeploy', 'terminate', 'start', 'stop', 'update']:
            args.append('-h')

    if debug:
        args.insert(1, '--debug')
    return args[1:]


def dispatch_cmds(args):
    if args.debug:
        requests_log = logging.getLogger("python-tutum")
        requests_log.setLevel(logging.INFO)
        cli_log = logging.getLogger("cli")
        cli_log.setLevel(logging.DEBUG)
    if args.cmd == 'login':
        commands.login()
    elif args.cmd == 'build':
        commands.build(args.tag, args.directory)
    elif args.cmd == 'service':
        if args.subcmd == 'create':
            commands.service_create(image=args.image, name=args.name, cpu_shares=args.cpushares,
                                    memory=args.memory, privileged=args.privileged,
                                    target_num_containers=args.target_num_containers, run_command=args.run_command,
                                    entrypoint=args.entrypoint, expose=args.expose, publish=args.publish,
                                    envvars=args.env, envfiles=args.env_file,
                                    tag=args.tag, linked_to_service=args.link_service,
                                    autorestart=args.autorestart, autodestroy=args.autodestroy,
                                    autoredeploy=args.autoredeploy, roles=args.role,
                                    sequential=args.sequential, volume=args.volume, volumes_from=args.volumes_from,
                                    deployment_strategy=args.deployment_strategy)
        elif args.subcmd == 'inspect':
            commands.service_inspect(args.identifier)
        elif args.subcmd == 'logs':
            commands.service_logs(args.identifier)
        elif args.subcmd == 'ps':
            commands.service_ps(args.quiet, args.status, args.stack)
        elif args.subcmd == 'redeploy':
            commands.service_redeploy(args.identifier)
        elif args.subcmd == 'run':
            commands.service_run(image=args.image, name=args.name, cpu_shares=args.cpushares,
                                 memory=args.memory, privileged=args.privileged,
                                 target_num_containers=args.target_num_containers, run_command=args.run_command,
                                 entrypoint=args.entrypoint, expose=args.expose, publish=args.publish, envvars=args.env,
                                 envfiles=args.env_file,
                                 tag=args.tag, linked_to_service=args.link_service,
                                 autorestart=args.autorestart, autodestroy=args.autodestroy,
                                 autoredeploy=args.autoredeploy, roles=args.role,
                                 sequential=args.sequential, volume=args.volume, volumes_from=args.volumes_from,
                                 deployment_strategy=args.deployment_strategy)
        elif args.subcmd == 'scale':
            commands.service_scale(args.identifier, args.target_num_containers)
        elif args.subcmd == 'set':
            commands.service_set(args.identifier, image=args.image, cpu_shares=args.cpushares,
                                 memory=args.memory, privileged=args.privileged,
                                 target_num_containers=args.target_num_containers, run_command=args.run_command,
                                 entrypoint=args.entrypoint, expose=args.expose, publish=args.publish, envvars=args.env,
                                 envfiles=args.env_file,
                                 tag=args.tag, linked_to_service=args.link_service,
                                 autorestart=args.autorestart, autodestroy=args.autodestroy,
                                 autoredeploy=args.autoredeploy, roles=args.role,
                                 sequential=args.sequential, redeploy=args.redeploy,
                                 volume=args.volume, volumes_from=args.volumes_from,
                                 deployment_strategy=args.deployment_strategy)
        elif args.subcmd == 'start':
            commands.service_start(args.identifier)
        elif args.subcmd == 'stop':
            commands.service_stop(args.identifier)
        elif args.subcmd == 'terminate':
            commands.service_terminate(args.identifier)
    elif args.cmd == 'container':
        if args.subcmd == 'inspect':
            commands.container_inspect(args.identifier)
        elif args.subcmd == 'logs':
            commands.container_logs(args.identifier)
        elif args.subcmd == 'ps':
            commands.container_ps(args.quiet, args.status, args.service)
        elif args.subcmd == 'start':
            commands.container_start(args.identifier)
        elif args.subcmd == 'stop':
            commands.container_stop(args.identifier)
        elif args.subcmd == 'terminate':
            commands.container_terminate(args.identifier)
    elif args.cmd == 'image':
        if args.subcmd == 'list':
            commands.image_list(args.quiet, args.jumpstarts, args.linux)
        elif args.subcmd == 'register':
            commands.image_register(args.image_name, args.description, args.username, args.password)
        elif args.subcmd == 'push':
            commands.image_push(args.name, args.public)
        elif args.subcmd == 'rm':
            commands.image_rm(args.image_name)
        elif args.subcmd == 'search':
            commands.image_search(args.query)
        elif args.subcmd == 'update':
            commands.image_update(args.image_name, args.username, args.password, args.description)
    elif args.cmd == 'node':
        if args.subcmd == 'inspect':
            commands.node_inspect(args.identifier)
        elif args.subcmd == 'list':
            commands.node_list(args.quiet)
        elif args.subcmd == 'rm':
            commands.node_rm(args.identifier)
        elif args.subcmd == 'upgrade':
            commands.node_upgrade(args.identifier)
        elif args.subcmd == 'byo':
            commands.node_byo()
    elif args.cmd == 'nodecluster':
        if args.subcmd == 'create':
            commands.nodecluster_create(args.target_num_nodes, args.name, args.provider, args.region, args.nodetype)
        elif args.subcmd == 'inspect':
            commands.nodecluster_inspect(args.identifier)
        elif args.subcmd == 'list':
            commands.nodecluster_list(args.quiet)
        elif args.subcmd == 'provider':
            commands.nodecluster_show_providers(args.quiet)
        elif args.subcmd == 'region':
            commands.nodecluster_show_regions(args.provider)
        elif args.subcmd == 'nodetype':
            commands.nodecluster_show_types(args.provider, args.region)
        elif args.subcmd == 'rm':
            commands.nodecluster_rm(args.identifier)
        elif args.subcmd == 'scale':
            commands.nodecluster_scale(args.identifier, args.target_num_nodes)
    elif args.cmd == 'tag':
        if args.subcmd == 'add':
            commands.tag_add(args.identifier, args.tag)
        elif args.subcmd == 'list':
            commands.tag_list(args.identifier, args.quiet)
        elif args.subcmd == 'rm':
            commands.tag_rm(args.identifier, args.tag)
        elif args.subcmd == 'set':
            commands.tag_set(args.identifier, args.tag)
    elif args.cmd == 'volume':
        if args.subcmd == 'list':
            commands.volume_list(args.quiet)
        if args.subcmd == 'inspect':
            commands.volume_inspect(args.identifier)
    elif args.cmd == 'volumegroup':
        if args.subcmd == 'list':
            commands.volumegroup_list(args.quiet)
        if args.subcmd == 'inspect':
            commands.volumegroup_inspect(args.identifier)
    elif args.cmd == 'webhook-handler':
        if args.subcmd == 'create':
            commands.webhookhandler_create(args.identifier, args.name)
        elif args.subcmd == 'list':
            commands.webhookhandler_list(args.identifier, args.quiet)
        elif args.subcmd == 'rm':
            commands.webhookhandler_rm(args.identifier, args.webhookhandler)
    elif args.cmd == 'stack':
        if args.subcmd == 'create':
            commands.stack_create(args.name, args.file)
        elif args.subcmd == 'inspect':
            commands.stack_inspect(args.identifier)
        elif args.subcmd == 'list':
            commands.stack_list(args.quiet)
        elif args.subcmd == 'redeploy':
            commands.stack_redeploy(args.identifier)
        elif args.subcmd == 'start':
            commands.stack_start(args.identifier)
        elif args.subcmd == 'stop':
            commands.stack_stop(args.identifier)
        elif args.subcmd == 'terminate':
            commands.stack_terminate(args.identifier)
        elif args.subcmd == 'up':
            commands.stack_up(args.name, args.file)
        elif args.subcmd == 'update':
            commands.stack_update(args.identifier, args.file)


def main():
    parser = initialize_parser()
    argv = patch_help_option(sys.argv)
    args = parser.parse_args(argv)
    commands.verify_auth(args)
    dispatch_cmds(args)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
import unittest
import __builtin__

import mock
from tutum.api.exceptions import *
import tutumcli
from tutumcli.utils import *
from tutumcli.exceptions import *


class TabulateResultTestCase(unittest.TestCase):
    @mock.patch('tutumcli.utils.tabulate')
    def test_tabulate_result(self, mock_tabulate):
        data_list = None
        headers = None
        tabulate_result(data_list, headers)
        mock_tabulate.assert_called_with(data_list, headers, stralign="left", tablefmt="plain")


class DateTimeConversionTestCase(unittest.TestCase):
    def test_from_utc_string_to_utc_datetime(self):
        # test None
        self.assertIsNone(from_utc_string_to_utc_datetime(None))

        # test mal-formatted string
        self.assertRaises(Exception, from_utc_string_to_utc_datetime, 'abc')

        # test normal case
        utc_datetime = from_utc_string_to_utc_datetime('Sun, 6 Apr 2014 18:11:17 +0000')
        self.assertEqual(str(utc_datetime), '2014-04-06 18:11:17')

    def test_get_humanize_local_datetime_from_utc_datetime_string(self):
        # test None
        self.assertEqual(get_humanize_local_datetime_from_utc_datetime_string(None), '')

        # test mal-formatted string
        self.assertRaises(Exception, get_humanize_local_datetime_from_utc_datetime_string, 'abc')

        # test normal case
        utc_datetime = get_humanize_local_datetime_from_utc_datetime_string('Sun, 6 Apr 2014 18:11:17 +0000')
        self.assertRegexpMatches(utc_datetime, r".* ago")

        # test future
        utc_datetime = get_humanize_local_datetime_from_utc_datetime_string('Sun, 6 Apr 3014 18:11:17 +0000')
        self.assertNotRegexpMatches(utc_datetime, r".* ago")


class IsUuidTestCase(unittest.TestCase):
    def test_is_uuid4(self):
        self.assertTrue(is_uuid4('7a4cfe51-038b-42d6-825e-3b533888d8cd'))
        self.assertTrue(is_uuid4('7A4CFE51-03BB-42D6-825E-3B533888D8CD'))

        self.assertFalse(is_uuid4('not_uuid'))
        self.assertFalse(is_uuid4(''))
        self.assertRaises(Exception, is_uuid4, None)
        self.assertRaises(Exception, is_uuid4, 12345)


class AddUnicodeSymbolToStateTestCase(unittest.TestCase):
    def test_add_unicode_symbol_to_state(self):
        for state in ['Running', 'Partly running']:
            self.assertEqual(' '.join([u'▶', state]), add_unicode_symbol_to_state(state))
        for state in ['Init', 'Stopped']:
            self.assertEqual(' '.join([u'◼', state]), add_unicode_symbol_to_state(state))
        for state in ['Starting', 'Stopping', 'Scaling', 'Terminating']:
            self.assertEqual(' '.join([u'⚙', state]), add_unicode_symbol_to_state(state))
        for state in ['Start failed', 'Stopped with errors']:
            self.assertEqual(' '.join([u'!', state]), add_unicode_symbol_to_state(state))
        for state in ['Terminated']:
            self.assertEqual(' '.join([u'✘', state]), add_unicode_symbol_to_state(state))


class GetDockerClientTestCase(unittest.TestCase):
    @mock.patch('tutumcli.utils.os.getenv')
    def test_get_docker_client_exception(self, mock_getenv):
        mock_getenv.return_value = '/run/mock.docker.sock'
        self.assertRaises(DockerNotFound, get_docker_client)


class FetchRemoteObjectTestCase(unittest.TestCase):
    @mock.patch('tutumcli.utils.tutum.Container.list')
    @mock.patch('tutumcli.utils.tutum.Container.fetch')
    def test_fetch_remote_container(self, mock_fetch, mock_list):
        # test container exist queried with uuid4
        mock_fetch.return_value = 'returned'
        self.assertEqual(fetch_remote_container('7A4CFE51-03BB-42D6-825E-3B533888D8CD', True), 'returned')
        self.assertEqual(fetch_remote_container('7A4CFE51-03BB-42D6-825E-3B533888D8CD', False), 'returned')

        # test container doesn't exist queried with uuid4
        mock_fetch.side_effect = ObjectNotFound
        self.assertRaises(ObjectNotFound, fetch_remote_container, '7A4CFE51-03BB-42D6-825E-3B533888D8CD', True)
        self.assertIsInstance(fetch_remote_container('7A4CFE51-03BB-42D6-825E-3B533888D8CD', False), ObjectNotFound)

        # test unique container found queried with short uuid
        container = tutum.Container.create()
        container.uuid = 'uuid'
        mock_list.side_effect = [[container], []]
        mock_fetch.side_effect = [container]
        self.assertEquals(fetch_remote_container('shortuuid', True), container)
        mock_list.side_effect = [[container], []]
        mock_fetch.side_effect = [container]
        self.assertEquals(fetch_remote_container('shortuuid', False), container)

        # test unique container found queried with name
        mock_list.side_effect = [[], [container]]
        mock_fetch.side_effect = [container]
        self.assertEquals(fetch_remote_container('name', True), container)
        mock_list.side_effect = [[], [container]]
        mock_fetch.side_effect = [container]
        self.assertEquals(fetch_remote_container('name', False), container)

        # test no container found
        mock_list.side_effect = [[], []]
        self.assertRaises(ObjectNotFound, fetch_remote_container, 'uuid_or_name', True)
        mock_list.side_effect = [[], []]
        self.assertIsInstance(fetch_remote_container('uuid_or_name', False), ObjectNotFound)

        # test multi-container found
        mock_list.side_effect = [['container1', 'container2'], []]
        self.assertRaises(NonUniqueIdentifier, fetch_remote_container, 'uuid_or_name', True)
        mock_list.side_effect = [['container1', 'container2'], []]
        self.assertIsInstance(fetch_remote_container('uuid_or_name', False), NonUniqueIdentifier)
        mock_list.side_effect = [[], ['container1', 'container2']]
        self.assertRaises(NonUniqueIdentifier, fetch_remote_container, 'uuid_or_name', True)
        mock_list.side_effect = [[], ['container1', 'container2']]
        self.assertIsInstance(fetch_remote_container('uuid_or_name', False), NonUniqueIdentifier)

        # test api error
        mock_list.side_effect = [TutumApiError, TutumApiError]
        self.assertRaises(TutumApiError, fetch_remote_container, 'uuid_or_name', True)
        self.assertRaises(TutumApiError, fetch_remote_container, 'uuid_or_name', False)

    @mock.patch('tutumcli.utils.tutum.Service.list')
    @mock.patch('tutumcli.utils.tutum.Service.fetch')
    def test_fetch_remote_service(self, mock_fetch, mock_list):
        # test cluster exist queried with uuid4
        mock_fetch.return_value = 'returned'
        self.assertEqual(fetch_remote_service('7A4CFE51-03BB-42D6-825E-3B533888D8CD', True), 'returned')
        self.assertEqual(fetch_remote_service('7A4CFE51-03BB-42D6-825E-3B533888D8CD', False), 'returned')

        # test cluster doesn't exist queried with uuid4
        mock_fetch.side_effect = ObjectNotFound
        self.assertRaises(ObjectNotFound, fetch_remote_service, '7A4CFE51-03BB-42D6-825E-3B533888D8CD', True)
        self.assertIsInstance(fetch_remote_service('7A4CFE51-03BB-42D6-825E-3B533888D8CD', False), ObjectNotFound)

        # test unique cluster found queried with short uuid
        service = tutum.Service.create()
        service.uuid = 'uuid'
        mock_list.side_effect = [[service], []]
        mock_fetch.side_effect = [service]
        self.assertEquals(fetch_remote_service('shortuuid', True), service)
        mock_list.side_effect = [[service], []]
        mock_fetch.side_effect = [service]
        self.assertEquals(fetch_remote_service('shortuuid', False), service)

        # test unique cluster found queried with name
        mock_list.side_effect = [[], [service]]
        mock_fetch.side_effect = [service]
        self.assertEquals(fetch_remote_service('name', True), service)
        mock_list.side_effect = [[], [service]]
        mock_fetch.side_effect = [service]
        self.assertEquals(fetch_remote_service('name', False), service)

        # test no cluster found
        mock_list.side_effect = [[], []]
        self.assertRaises(ObjectNotFound, fetch_remote_service, 'uuid_or_name', True)
        mock_list.side_effect = [[], []]
        self.assertIsInstance(fetch_remote_service('uuid_or_name', False), ObjectNotFound)

        # test multi-cluster found
        mock_list.side_effect = [['cluster1', 'cluster2'], []]
        self.assertRaises(NonUniqueIdentifier, fetch_remote_service, 'uuid_or_name', True)
        mock_list.side_effect = [['cluster1', 'cluster2'], []]
        self.assertIsInstance(fetch_remote_service('uuid_or_name', False), NonUniqueIdentifier)
        mock_list.side_effect = [[], ['cluster1', 'cluster2']]
        self.assertRaises(NonUniqueIdentifier, fetch_remote_service, 'uuid_or_name', True)
        mock_list.side_effect = [[], ['cluster1', 'cluster2']]
        self.assertIsInstance(fetch_remote_service('uuid_or_name', False), NonUniqueIdentifier)

        # test api error
        mock_list.side_effect = [TutumApiError, TutumApiError]
        self.assertRaises(TutumApiError, fetch_remote_service, 'uuid_or_name', True)
        self.assertRaises(TutumApiError, fetch_remote_service, 'uuid_or_name', False)

    @mock.patch('tutumcli.utils.tutum.Node.list')
    @mock.patch('tutumcli.utils.tutum.Node.fetch')
    def test_fetch_remote_node(self, mock_fetch, mock_list):
        # test node exist queried with uuid4
        mock_fetch.return_value = 'returned'
        self.assertEqual(fetch_remote_node('7A4CFE51-03BB-42D6-825E-3B533888D8CD', True), 'returned')
        self.assertEqual(fetch_remote_node('7A4CFE51-03BB-42D6-825E-3B533888D8CD', False), 'returned')

        # test node doesn't exist queried with uuid4
        mock_fetch.side_effect = ObjectNotFound
        self.assertRaises(ObjectNotFound, fetch_remote_node, '7A4CFE51-03BB-42D6-825E-3B533888D8CD', True)
        self.assertIsInstance(fetch_remote_node('7A4CFE51-03BB-42D6-825E-3B533888D8CD', False), ObjectNotFound)


        # test unique node found queried with short uuid
        node = tutum.Node.create()
        node.uuid = 'uuid'
        mock_list.side_effect = [[node]]
        mock_fetch.side_effect = [node]
        self.assertEquals(fetch_remote_node('uuid', True), node)
        mock_list.side_effect = [[node]]
        mock_fetch.side_effect = [node]
        self.assertEquals(fetch_remote_node('uuid', False), node)

        # test no node found
        mock_list.side_effect = [[]]
        self.assertRaises(ObjectNotFound, fetch_remote_node, 'uuid', True)
        mock_list.side_effect = [[]]
        self.assertIsInstance(fetch_remote_node('uuid', False), ObjectNotFound)

        # test multi-node found
        mock_list.side_effect = [['node1', 'node2']]
        self.assertRaises(NonUniqueIdentifier, fetch_remote_node, 'uuid', True)
        mock_list.side_effect = [['node1', 'node2']]
        self.assertIsInstance(fetch_remote_node('uuid', False), NonUniqueIdentifier)

        # test api error
        mock_list.side_effect = [TutumApiError, TutumApiError]
        self.assertRaises(TutumApiError, fetch_remote_node, 'uuid', True)
        self.assertRaises(TutumApiError, fetch_remote_node, 'uuid', False)

    @mock.patch('tutumcli.utils.tutum.NodeCluster.list')
    @mock.patch('tutumcli.utils.tutum.NodeCluster.fetch')
    def test_fetch_remote_nodecluster(self, mock_fetch, mock_list):
        # test nodecluster exist queried with uuid4
        mock_fetch.return_value = 'returned'
        self.assertEqual(fetch_remote_nodecluster('7A4CFE51-03BB-42D6-825E-3B533888D8CD', True), 'returned')
        self.assertEqual(fetch_remote_nodecluster('7A4CFE51-03BB-42D6-825E-3B533888D8CD', False), 'returned')

        # test nodecluster doesn't exist queried with uuid4
        mock_fetch.side_effect = ObjectNotFound
        self.assertRaises(ObjectNotFound, fetch_remote_nodecluster, '7A4CFE51-03BB-42D6-825E-3B533888D8CD', True)
        self.assertIsInstance(fetch_remote_nodecluster('7A4CFE51-03BB-42D6-825E-3B533888D8CD', False), ObjectNotFound)

        # test unique nodecluster found queried with short uuid
        nodecluster = tutum.NodeCluster.create()
        nodecluster.uuid = 'uuid'
        mock_list.side_effect = [[nodecluster], []]
        mock_fetch.side_effect = [nodecluster]
        self.assertEquals(fetch_remote_nodecluster('shortuuid', True), nodecluster)
        mock_list.side_effect = [[nodecluster], []]
        mock_fetch.side_effect = [nodecluster]
        self.assertEquals(fetch_remote_nodecluster('shortuuid', False), nodecluster)

        # test unique nodecluster found queried with name
        mock_list.side_effect = [[], [nodecluster]]
        mock_fetch.side_effect = [nodecluster]
        self.assertEquals(fetch_remote_nodecluster('name', True), nodecluster)
        mock_list.side_effect = [[], [nodecluster]]
        mock_fetch.side_effect = [nodecluster]
        self.assertEquals(fetch_remote_nodecluster('name', False), nodecluster)

        # test no nodecluster found
        mock_list.side_effect = [[], []]
        self.assertRaises(ObjectNotFound, fetch_remote_nodecluster, 'uuid_or_name', True)
        mock_list.side_effect = [[], []]
        self.assertIsInstance(fetch_remote_nodecluster('uuid_or_name', False), ObjectNotFound)

        # test multi-nodecluster found
        mock_list.side_effect = [['nodecluster1', 'nodecluster2'], []]
        self.assertRaises(NonUniqueIdentifier, fetch_remote_nodecluster, 'uuid_or_name', True)
        mock_list.side_effect = [['nodecluster1', 'nodecluster2'], []]
        self.assertIsInstance(fetch_remote_nodecluster('uuid_or_name', False), NonUniqueIdentifier)
        mock_list.side_effect = [[], ['nodecluster1', 'nodecluster2']]
        self.assertRaises(NonUniqueIdentifier, fetch_remote_nodecluster, 'uuid_or_name', True)
        mock_list.side_effect = [[], ['nodecluster1', 'nodecluster2']]
        self.assertIsInstance(fetch_remote_nodecluster('uuid_or_name', False), NonUniqueIdentifier)

        # test api error
        mock_list.side_effect = [TutumApiError, TutumApiError]
        self.assertRaises(TutumApiError, fetch_remote_nodecluster, 'uuid_or_name', True)
        self.assertRaises(TutumApiError, fetch_remote_nodecluster, 'uuid_or_name', False)


class ParseLinksTestCase(unittest.TestCase):
    def test_parse_links(self):
        output = [{'to_service': 'mysql', 'name': 'db1'}, {'to_service': 'mariadb', 'name': 'db2'}]
        self.assertEqual(output, parse_links(['mysql:db1', 'mariadb:db2'], 'to_service'))

    def test_parse_links_bad_format(self):
        self.assertRaises(BadParameter, parse_links, ['mysql', 'mariadb'], 'to_service')
        self.assertRaises(BadParameter, parse_links, ['mysql:mysql:mysql', 'mariadb:maria:maria'], 'to_service')
        self.assertRaises(BadParameter, parse_links, [''], 'to_service')


class ParsePublishedPortsTestCase(unittest.TestCase):
    def test_parse_published_ports(self):
        output = [{'protocol': 'tcp', 'inner_port': '80', 'published': True},
                  {'protocol': 'udp', 'inner_port': '53', 'published': True},
                  {'protocol': 'tcp', 'inner_port': '3306', 'outer_port': '3307', 'published': True},
                  {'protocol': 'udp', 'inner_port': '8080', 'outer_port': '8083', 'published': True}]
        self.assertEqual(output, parse_published_ports(['80', '53/udp', '3307:3306', '8083:8080/udp']))

    def test_parse_published_ports_bad_format(self):
        self.assertRaises(BadParameter, parse_published_ports, ['abc'])
        self.assertRaises(BadParameter, parse_published_ports, ['abc:80'])
        self.assertRaises(BadParameter, parse_published_ports, ['80:abc'])
        self.assertRaises(BadParameter, parse_published_ports, ['80:80:abc'])
        self.assertRaises(BadParameter, parse_published_ports, ['80:80/abc'])
        self.assertRaises(BadParameter, parse_published_ports, ['80/80:tcp'])
        self.assertRaises(BadParameter, parse_published_ports, [''])


class ParseExposedPortsTestCase(unittest.TestCase):
    def test_parse_exposed_ports(self):
        output = [{'protocol': 'tcp', 'inner_port': '80', 'published': False},
                  {'protocol': 'tcp', 'inner_port': '8080', 'published': False}]
        self.assertEqual(output, parse_exposed_ports([80, 8080]))

    def test_parse_exposed_ports_bad_format(self):
        self.assertRaises(BadParameter, parse_exposed_ports, ['abc'])
        self.assertRaises(BadParameter, parse_exposed_ports, ['abc'])
        self.assertRaises(BadParameter, parse_exposed_ports, ['-1'])
        self.assertRaises(BadParameter, parse_exposed_ports, ['999999'])


class ParseEnvironmentVariablesTestCase(unittest.TestCase):
    def test_parse_envvars(self):
        output = [{'key': 'MYSQL_PASS', 'value': 'mypass'}, {'key': 'MYSQL_USER', 'value': 'admin'}]
        self.assertEqual(output, parse_envvars(['MYSQL_USER=admin', 'MYSQL_PASS=mypass'], []))


class TryRegisterTestCase(unittest.TestCase):
    def setUp(self):
        self.raw_input_holder = __builtin__.raw_input

    def tearDown(self):
        __builtin__.raw_input = self.raw_input_holder


    @mock.patch('tutumcli.utils.requests.post')
    def test_try_register_success(self, mock_post):
        username = 'test_username'
        password = 'test_password'
        __builtin__.raw_input = lambda _: 'test@email.com'  # set email
        response = mock.MagicMock()
        response.status_code = 201
        mock_post.return_value = response
        url = urlparse.urljoin(tutum.base_url, "register/")
        headers = {'Content-Type': "application/json", "User-Agent": "tutum/%s" % tutumcli.__version__}
        data = json.dumps({'username': 'test_username', "password1": 'test_password', "password2": 'test_password',
                           "email": 'test@email.com'})

        ret, text = try_register(username, password)
        mock_post.assert_called_with(url, headers=headers, data=data)
        self.assertEqual((True, ('Account created. Please check your email for activation instructions.')), (ret, text))

    @mock.patch('tutumcli.utils.requests.post')
    def test_try_register_too_many_retries(self, mock_post):
        username = 'test_username'
        password = 'test_password'
        __builtin__.raw_input = lambda _: 'test@email.com'  # set email
        response = mock.MagicMock()
        response.status_code = 429
        mock_post.return_value = response
        ret, text = try_register(username, password)
        self.assertEqual((False, "Too many retries. Please login again later."), (ret, text))

    @mock.patch('tutumcli.utils.requests.post')
    def test_try_register_failed(self, mock_post):
        username = 'test_username'
        password = 'test_password'
        __builtin__.raw_input = lambda _: 'test@email.com'  # set email
        response = mock.Mock()
        response.status_code = 400
        response.json.return_value = {u'register': {u'username': [u'A user with that username already exists.'],
                                                    u'email': [
                                                        u'This email address is already in use. Please supply a different email address.']}}
        mock_post.return_value = response

        ret, text = try_register(username, password)
        self.assertEqual((False,
                          u'username: A user with that username already exists.\nemail: This email address is already in use. Please supply a different email address.'),
                         (ret, text))



from __future__ import print_function
import datetime
import json
import urlparse
import ssl
import re
import os
import codecs

import yaml
from tabulate import tabulate
import tutum
from dateutil import tz
import ago
import docker
import requests
from tutumcli.exceptions import NonUniqueIdentifier, ObjectNotFound, BadParameter, DockerNotFound
from exceptions import StreamOutputError
from . import __version__


def tabulate_result(data_list, headers):
    print(tabulate(data_list, headers, stralign="left", tablefmt="plain"))


def from_utc_string_to_utc_datetime(utc_datetime_string):
    if not utc_datetime_string:
        return None
    utc_date_object = datetime.datetime.strptime(utc_datetime_string, "%a, %d %b %Y %H:%M:%S +0000")

    return utc_date_object


def get_humanize_local_datetime_from_utc_datetime_string(utc_datetime_string):
    def get_humanize_local_datetime_from_utc_datetime(utc_target_datetime):
        local_now = datetime.datetime.now(tz.tzlocal())
        if utc_target_datetime:
            local_target_datetime = utc_target_datetime.replace(tzinfo=tz.gettz("UTC")).astimezone(tz=tz.tzlocal())
            return ago.human(local_now - local_target_datetime, precision=1)
        return ""

    utc_target_datetime = from_utc_string_to_utc_datetime(utc_datetime_string)
    return get_humanize_local_datetime_from_utc_datetime(utc_target_datetime)


def is_uuid4(identifier):
    uuid4_regexp = re.compile('^[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12}', re.I)
    match = uuid4_regexp.match(identifier)
    return bool(match)


def add_unicode_symbol_to_state(state):
    if state in ["Running", "Partly running", "Deployed"]:
        return u"\u25B6 " + state
    elif state in ["Init", "Stopped"]:
        return u"\u25FC " + state
    elif state in ["Starting", "Stopping", "Scaling", "Terminating", "Deploying", "Redeploying"]:
        return u"\u2699 " + state
    elif state in ["Start failed", "Stopped with errors"]:
        return u"\u0021 " + state
    elif state == "Terminated":
        return u"\u2718 " + state
    elif state == "Unreachable":
        return u"\u2753 " + state
    return state


def get_docker_client():
    try:
        DOCKER_TLS_VERIFY = bool(os.environ.get('DOCKER_TLS_VERIFY', False))

        if not DOCKER_TLS_VERIFY:
            tls_config = False
        else:
            cert_path = os.environ.get('DOCKER_CERT_PATH')
            if cert_path:
                ca_cert_path = os.path.join(cert_path, "ca.pem")
                client_cert = (os.path.join(cert_path, "cert.pem"), os.path.join(cert_path, "key.pem"))
                tls_config = docker.tls.TLSConfig(
                    ssl_version=ssl.PROTOCOL_TLSv1,
                    client_cert=client_cert,
                    verify=ca_cert_path,
                    assert_hostname=False)
            else:
                tls_config = False

        base_url = os.getenv("DOCKER_HOST", None)
        if tls_config and base_url.startswith("tcp://"):
            base_url = base_url.replace("tcp://", "https://")

        docker_client = docker.Client(base_url=base_url, tls=tls_config, version='auto')
        docker_client.version()
        return docker_client
    except Exception as e:
        raise DockerNotFound("Cannot connect to docker (is it running?)")


def stream_output(output, stream):
    def print_output_event(event, stream, is_terminal):
        if 'errorDetail' in event:
            raise StreamOutputError(event['errorDetail']['message'])

        terminator = ''

        if is_terminal and 'stream' not in event:
            # erase current line
            stream.write("%c[2K\r" % 27)
            terminator = "\r"
            pass
        elif 'progressDetail' in event:
            return

        if 'time' in event:
            stream.write("[%s] " % event['time'])

        if 'id' in event:
            stream.write("%s: " % event['id'])

        if 'from' in event:
            stream.write("(from %s) " % event['from'])

        status = event.get('status', '')

        if 'progress' in event:
            stream.write("%s %s%s" % (status, event['progress'], terminator))
        elif 'progressDetail' in event:
            detail = event['progressDetail']
            if 'current' in detail:
                percentage = float(detail['current']) / float(detail['total']) * 100
                stream.write('%s (%.1f%%)%s' % (status, percentage, terminator))
            else:
                stream.write('%s%s' % (status, terminator))
        elif 'stream' in event:
            stream.write("%s%s" % (event['stream'], terminator))
        else:
            stream.write("%s%s\n" % (status, terminator))

    is_terminal = hasattr(stream, 'fileno') and os.isatty(stream.fileno())
    stream = codecs.getwriter('utf-8')(stream)
    all_events = []
    lines = {}
    diff = 0

    for chunk in output:
        event = json.loads(chunk)
        all_events.append(event)

        if 'progress' in event or 'progressDetail' in event:
            image_id = event.get('id')
            if not image_id:
                continue

            if image_id in lines:
                diff = len(lines) - lines[image_id]
            else:
                lines[image_id] = len(lines)
                stream.write("\n")
                diff = 0

            if is_terminal:
                # move cursor up `diff` rows
                stream.write("%c[%dA" % (27, diff))

        print_output_event(event, stream, is_terminal)

        if 'id' in event and is_terminal:
            # move cursor back down
            stream.write("%c[%dB" % (27, diff))

        stream.flush()

    return all_events


def fetch_remote_container(identifier, raise_exceptions=True):
    try:
        if is_uuid4(identifier):
            try:
                return tutum.Container.fetch(identifier)
            except Exception:
                raise ObjectNotFound("Cannot find a container with the identifier '%s'" % identifier)
        else:
            objects_same_identifier = tutum.Container.list(uuid__startswith=identifier) or \
                                      tutum.Container.list(name=identifier)
            if len(objects_same_identifier) == 1:
                uuid = objects_same_identifier[0].uuid
                return tutum.Container.fetch(uuid)
            elif len(objects_same_identifier) == 0:
                raise ObjectNotFound("Cannot find a container with the identifier '%s'" % identifier)
            raise NonUniqueIdentifier("More than one container has the same identifier, please use the long uuid")

    except (NonUniqueIdentifier, ObjectNotFound) as e:
        if not raise_exceptions:
            return e
        raise e


def fetch_remote_service(identifier, raise_exceptions=True):
    try:
        if is_uuid4(identifier):
            try:
                return tutum.Service.fetch(identifier)
            except Exception:
                raise ObjectNotFound("Cannot find a service with the identifier '%s'" % identifier)
        else:
            objects_same_identifier = tutum.Service.list(uuid__startswith=identifier) or \
                                      tutum.Service.list(name=identifier)

            if len(objects_same_identifier) == 1:
                uuid = objects_same_identifier[0].uuid
                return tutum.Service.fetch(uuid)
            elif len(objects_same_identifier) == 0:
                raise ObjectNotFound("Cannot find a service with the identifier '%s'" % identifier)
            raise NonUniqueIdentifier("More than one service has the same identifier, please use the long uuid")
    except (NonUniqueIdentifier, ObjectNotFound) as e:
        if not raise_exceptions:
            return e
        raise e


def fetch_remote_stack(identifier, raise_exceptions=True):
    try:
        if is_uuid4(identifier):
            try:
                return tutum.Stack.fetch(identifier)
            except Exception:
                raise ObjectNotFound("Cannot find a stack with the identifier '%s'" % identifier)
        else:
            objects_same_identifier = tutum.Stack.list(uuid__startswith=identifier) or \
                                      tutum.Stack.list(name=identifier)
            if len(objects_same_identifier) == 1:
                uuid = objects_same_identifier[0].uuid
                return tutum.Stack.fetch(uuid)
            elif len(objects_same_identifier) == 0:
                raise ObjectNotFound("Cannot find a stack with the identifier '%s'" % identifier)
            raise NonUniqueIdentifier("More than one stack has the same identifier, please use the long uuid")

    except (NonUniqueIdentifier, ObjectNotFound) as e:
        if not raise_exceptions:
            return e
        raise e


def fetch_remote_volume(identifier, raise_exceptions=True):
    try:
        if is_uuid4(identifier):
            try:
                return tutum.Volume.fetch(identifier)
            except Exception:
                raise ObjectNotFound("Cannot find a volume with the identifier '%s'" % identifier)
        else:
            objects_same_identifier = tutum.Volume.list(uuid__startswith=identifier)
            if len(objects_same_identifier) == 1:
                uuid = objects_same_identifier[0].uuid
                return tutum.Volume.fetch(uuid)
            elif len(objects_same_identifier) == 0:
                raise ObjectNotFound("Cannot find a volume with the identifier '%s'" % identifier)
            raise NonUniqueIdentifier("More than one volume has the same identifier, please use the long uuid")

    except (NonUniqueIdentifier, ObjectNotFound) as e:
        if not raise_exceptions:
            return e
        raise e


def fetch_remote_volumegroup(identifier, raise_exceptions=True):
    try:
        if is_uuid4(identifier):
            try:
                return tutum.VolumeGroup.fetch(identifier)
            except Exception:
                raise ObjectNotFound("Cannot find a volume with the identifier '%s'" % identifier)
        else:
            objects_same_identifier = tutum.VolumeGroup.list(uuid__startswith=identifier) or \
                                      tutum.VolumeGroup.list(name=identifier)
            if len(objects_same_identifier) == 1:
                uuid = objects_same_identifier[0].uuid
                return tutum.VolumeGroup.fetch(uuid)
            elif len(objects_same_identifier) == 0:
                raise ObjectNotFound("Cannot find a volume with the identifier '%s'" % identifier)
            raise NonUniqueIdentifier("More than one volume has the same identifier, please use the long uuid")

    except (NonUniqueIdentifier, ObjectNotFound) as e:
        if not raise_exceptions:
            return e
        raise e


def fetch_remote_node(identifier, raise_exceptions=True):
    try:
        if is_uuid4(identifier):
            try:
                return tutum.Node.fetch(identifier)
            except Exception:
                raise ObjectNotFound("Cannot find a node with the identifier '%s'" % identifier)
        else:
            objects_same_identifier = tutum.Node.list(uuid__startswith=identifier)
            if len(objects_same_identifier) == 1:
                uuid = objects_same_identifier[0].uuid
                return tutum.Node.fetch(uuid)
            elif len(objects_same_identifier) == 0:
                raise ObjectNotFound("Cannot find a node with the identifier '%s'" % identifier)
            raise NonUniqueIdentifier("More than one node has the same identifier, please use the long uuid")

    except (NonUniqueIdentifier, ObjectNotFound) as e:
        if not raise_exceptions:
            return e
        raise e


def fetch_remote_nodecluster(identifier, raise_exceptions=True):
    try:
        if is_uuid4(identifier):
            try:
                return tutum.NodeCluster.fetch(identifier)
            except Exception:
                raise ObjectNotFound("Cannot find a node cluster with the identifier '%s'" % identifier)
        else:
            objects_same_identifier = tutum.NodeCluster.list(uuid__startswith=identifier) or \
                                      tutum.NodeCluster.list(name=identifier)
            if len(objects_same_identifier) == 1:
                uuid = objects_same_identifier[0].uuid
                return tutum.NodeCluster.fetch(uuid)
            elif len(objects_same_identifier) == 0:
                raise ObjectNotFound("Cannot find a node cluster with the identifier '%s'" % identifier)
            raise NonUniqueIdentifier("More than one node cluster has the same identifier, please use the long uuid")

    except (NonUniqueIdentifier, ObjectNotFound) as e:
        if not raise_exceptions:
            return e
        raise e


def get_uuids_of_webhookhandler(webhookhandler, identifiers):
    uuid_list = []
    for identifier in identifiers:
        if is_uuid4(identifier):
            uuid_list.append(identifier)
        else:
            handlers = webhookhandler.list(uuid__startswith=identifier) or \
                       webhookhandler.list(name=identifier)
            for handler in handlers:
                uuid = handler.get('uuid', "")
                if uuid:
                    uuid_list.append(uuid)
    if not uuid_list:
        raise ObjectNotFound("Cannot find a webhook handler with the identifier '%s'" % identifier)
    return uuid_list


def parse_links(links, target):
    def _format_link(_link):
        link_regexp = re.compile('^[a-zA-Z0-9_-]+:[a-zA-Z0-9_-]+$')
        match = link_regexp.match(_link)
        if match:
            temp = _link.split(":", 1)
            return {target: temp[0], 'name': temp[1]}
        raise BadParameter("Link variable argument %s does not match with (name:alias)."
                           " Example: mysql:db" % _link)

    return [_format_link(link) for link in links] if links else []


def parse_published_ports(port_list):
    def _get_port_dict(_port):
        port_regexp = re.compile('^([0-9]{1,5}:)?([0-9]{1,5})(/tcp|/udp)?$')
        match = port_regexp.match(_port)
        if bool(match):
            outer_port = match.group(1)
            inner_port = match.group(2)
            protocol = match.group(3)
            if protocol is None:
                protocol = "tcp"
            else:
                protocol = protocol[1:]

            port_spec = {'protocol': protocol, 'inner_port': inner_port, 'published': True}

            if outer_port is not None:
                port_spec['outer_port'] = outer_port[:-1]
            return port_spec
        raise BadParameter("publish port %s does not match with '[host_port:]container_port[/protocol]'."
                           " E.g: 80:80/tcp" % _port)

    parsed_ports = []
    if port_list is not None:
        parsed_ports = []
        for port in port_list:
            parsed_ports.append(_get_port_dict(port))
    return parsed_ports


def parse_exposed_ports(port_list):
    def _get_port_dict(_port):
        if isinstance(_port, int) and 0 <= _port < 65535:
            port_spec = {'protocol': 'tcp', 'inner_port': '%d' % _port, 'published': False}
            return port_spec
        raise BadParameter("expose port %s is not a valid port number" % _port)

    parsed_ports = []
    if port_list is not None:
        parsed_ports = []
        for port in port_list:
            parsed_ports.append(_get_port_dict(port))
    return parsed_ports


def parse_envvars(envvar_list, envfile_list):
    def _transform_envvar(_envvar):
        _envvar = _envvar.split("=", 1)
        length = len(_envvar)
        if length == 2:
            return {'key': _envvar[0], 'value': _envvar[1]}
        else:
            raise BadParameter("Environment variable '%s' does not match with 'KEY=VALUE'."
                               " Example: ENVVAR=foo" % _envvar[0])

    def _read_envvar(envfile):
        envvars = []
        with open(envfile) as f:
            lines = f.readlines()
            for line in lines:
                line = line.strip()
                if line.startswith("#"):
                    continue
                if line == "":
                    continue
                envvars.append(line)
            return envvars

    transformed_envvars = []
    envvars = []
    if envfile_list is not None:
        for envfile in envfile_list:
            envvars.extend(_read_envvar(envfile))

    if envvar_list is not None:
        envvars.extend(envvar_list)

    if envvars is not None:
        for envvar in envvars:
            transformed_envvars.append(_transform_envvar(envvar))

    parsed_envvar_dict = {}
    parsed_envvar_list = []
    for transformed_envvar in transformed_envvars:
        parsed_envvar_dict[transformed_envvar["key"]] = transformed_envvar
    for v in parsed_envvar_dict.itervalues():
        parsed_envvar_list.append(v)

    return parsed_envvar_list


def try_register(username, password):
    email = raw_input("Email: ")

    headers = {"Content-Type": "application/json", "User-Agent": "tutum/%s" % __version__}
    data = {'username': username, "password1": password, "password2": password, "email": email}

    try:
        r = requests.post(urlparse.urljoin(tutum.base_url, "register/"), data=json.dumps(data), headers=headers)
        if r.status_code == 201:
            return True, "Account created. Please check your email for activation instructions."
        elif r.status_code == 429:
            return False, "Too many retries. Please login again later."
        else:
            messages = r.json()['register']
            if isinstance(messages, dict):
                _text = []
                for key in messages.keys():
                    _text.append("%s: %s" % (key, '\n'.join(messages[key])))
                _text = '\n'.join(_text)
            else:
                _text = messages
            return False, _text
    except Exception:
        return False, r.text


def parse_volume(volume):
    bindings = []
    if not volume:
        return bindings

    for vol in volume:
        binding = {}
        terms = vol.split(":")
        if len(terms) == 1:
            binding["container_path"] = terms[0]
        elif len(terms) == 2:
            binding["host_path"] = terms[0]
            binding["container_path"] = terms[1]
        elif len(terms) == 3:
            binding["host_path"] = terms[0]
            binding["container_path"] = terms[1]
            if terms[2].lower() == 'ro':
                binding["rewritable"] = False
        else:
            raise BadParameter('Bad volume argument %s. Format: "[host_path:]/container_path[:permission]"' % vol)
        bindings.append(binding)
    return bindings


def parse_volumes_from(volumes_from):
    bindings = []
    if not volumes_from:
        return bindings

    for identifier in volumes_from:
        binding = {}
        service = fetch_remote_service(identifier)
        binding["volumes_from"] = service.resource_uri
        bindings.append(binding)
    return bindings


def loadStackFile(name, stackfile, stack=None):
    if not stack:
        stack = tutum.Stack.create()
    else:
        name = stack.name

    if not stackfile:
        filematch = 0
        if os.path.exists("tutum.yml"):
            stackfile = "tutum.yml"
            filematch += 1
        if os.path.exists("tutum.yaml"):
            stackfile = "tutum.yaml"
            filematch += 1
        if os.path.exists("tutum.json"):
            stackfile = "tutum.json"
            filematch += 1
        if filematch == 0:
            raise BadParameter("Cannot find Stackfile. Are you in the right directory?")
        elif filematch > 1:
            raise BadParameter("More than one Stackfile was found in the path. "
                               "Please specify which one you'd like to use with -f <filename>")
    with open(stackfile, 'r') as f:
        content = yaml.load(f.read())
        service = []
        if content:
            for k, v in content.items():
                v.update({"name": k})
                service.append(v)

            if not name:
                name = os.path.basename(os.getcwd())
            data = {'name': name, 'services': service}
            for k, v in list(data.items()):
                setattr(stack, k, v)
        else:
            raise BadParameter("Bad format of the stackfile: %s" % stackfile)
    return stack


import unittest
import copy
import StringIO
import sys

import mock
from tutumcli.tutum_cli import patch_help_option, dispatch_cmds, initialize_parser
from tutumcli.exceptions import InternalError
import tutumcli


class PatchHelpOptionTestCase(unittest.TestCase):
    def setUp(self):
        self.add_help_argv_list = [
            ['tutum'],
            ['tutum', 'service'],
            ['tutum', 'service', 'create'],
            ['tutum', 'service', 'inspect'],
            ['tutum', 'service', 'logs'],
            ['tutum', 'service', 'redeploy'],
            ['tutum', 'service', 'run'],
            ['tutum', 'service', 'scale'],
            ['tutum', 'service', 'set'],
            ['tutum', 'service', 'start'],
            ['tutum', 'service', 'stop'],
            ['tutum', 'service', 'terminate'],
            ['tutum', 'build'],
            ['tutum', 'container'],
            ['tutum', 'container', 'inspect'],
            ['tutum', 'container', 'logs'],
            ['tutum', 'container', 'start'],
            ['tutum', 'container', 'stop'],
            ['tutum', 'container', 'terminate'],
            ['tutum', 'image'],
            ['tutum', 'image', 'register'],
            ['tutum', 'image', 'push'],
            ['tutum', 'image', 'rm'],
            ['tutum', 'image', 'search'],
            ['tutum', 'image', 'update'],
            ['tutum', 'node'],
            ['tutum', 'node', 'inspect'],
            ['tutum', 'node', 'rm'],
            ['tutum', 'node', 'upgrade'],
            ['tutum', 'nodecluster'],
            ['tutum', 'nodecluster', 'create'],
            ['tutum', 'nodecluster', 'inspect'],
            ['tutum', 'nodecluster', 'rm'],
            ['tutum', 'nodecluster', 'scale'],
            ['tutum', 'tag', 'add'],
            ['tutum', 'tag', 'list'],
            ['tutum', 'tag', 'rm'],
            ['tutum', 'tag', 'set'],
        ]
        self.not_add_help_argv_list = [
            ["tutum", "service", "ps"],
            ["tutum", "container", "ps"],
            ["tutum", "image", "list"],
            ["tutum", "node", "list"],
            ["tutum", "nodecluster", "list"],
            ["tutum", "nodecluster", "provider"],
            ["tutum", "nodecluster", "region"],
            ['tutum', 'nodecluster', 'nodetype'],
            ["tutum", "container", "run", "-p", "80:80", "tutum/wordpress"],
        ]

    def test_parser_with_empty_args(self):
        args = []
        self.assertRaises(InternalError, patch_help_option, args)

    def test_help_append(self):
        for argv in self.add_help_argv_list:
            args = patch_help_option(argv)
            target = copy.copy(argv[1:])
            target.append('-h')
            self.assertEqual(target, args, "Help option not patch correctly: %s" % argv)

    def test_help_not_append(self):
        for argv in self.not_add_help_argv_list:
            args = patch_help_option(argv)
            self.assertEqual(argv[1:], args, "Should not patch help option correctly: %s" % argv)

    def test_help_append_with_debug_option(self):
        argvlist = copy.copy(self.add_help_argv_list)
        for argv in argvlist:
            argv.insert(1, "--debug")
            args = patch_help_option(argv)
            target = copy.copy(argv[1:])
            target.append('-h')
            self.assertEqual(target, args, "Help option not patch correctly: %s" % argv)

    def test_help_not_append_with_debug_option(self):
        argvlist = copy.copy(self.not_add_help_argv_list)
        for argv in argvlist:
            argv.insert(1, "--debug")
            args = patch_help_option(argv)
            self.assertEqual(argv[1:], args, "Should not patch help option correctly: %s" % argv)


class CommandsDispatchTestCase(unittest.TestCase):
    def setUp(self):
        self.parser = tutumcli.tutum_cli.initialize_parser()

    @mock.patch('tutumcli.tutum_cli.commands')
    def test_login_dispatch(self, mock_cmds):
        args = self.parser.parse_args(['login'])
        dispatch_cmds(args)
        mock_cmds.login.assert_called_with()

    @mock.patch('tutumcli.tutum_cli.commands')
    def test_build_dispatch(self, mock_cmds):
        args = self.parser.parse_args(['build', '-t', 'mysql', '.'])
        dispatch_cmds(args)
        mock_cmds.build.assert_called_with(args.tag, args.directory)

    @mock.patch('tutumcli.tutum_cli.commands')
    def test_service_dispatch(self, mock_cmds):
        args = self.parser.parse_args(['service', 'create', 'mysql'])
        dispatch_cmds(args)
        mock_cmds.service_create.assert_called_with(image=args.image, name=args.name, cpu_shares=args.cpushares,
                                                    memory=args.memory,
                                                    target_num_containers=args.target_num_containers,
                                                    privileged=args.privileged,
                                                    run_command=args.run_command,
                                                    entrypoint=args.entrypoint, expose=args.expose,
                                                    publish=args.publish,
                                                    envvars=args.env, envfiles=args.env_file, tag=args.tag,
                                                    linked_to_service=args.link_service,
                                                    autorestart=args.autorestart, autodestroy=args.autodestroy,
                                                    autoredeploy=args.autoredeploy, roles=args.role,
                                                    sequential=args.sequential,
                                                    volume=args.volume, volumes_from=args.volumes_from,
                                                    deployment_strategy=args.deployment_strategy)

        args = self.parser.parse_args(['service', 'inspect', 'id'])
        dispatch_cmds(args)
        mock_cmds.service_inspect.assert_called_with(args.identifier)

        args = self.parser.parse_args(['service', 'logs', 'id'])
        dispatch_cmds(args)
        mock_cmds.service_logs.assert_called_with(args.identifier)

        args = self.parser.parse_args(['service', 'ps'])
        dispatch_cmds(args)
        mock_cmds.service_ps.assert_called_with(args.quiet, args.status, args.stack)

        args = self.parser.parse_args(['service', 'redeploy', 'mysql'])
        dispatch_cmds(args)
        mock_cmds.service_redeploy.assert_called_with(args.identifier)

        args = self.parser.parse_args(['service', 'run', 'mysql'])
        dispatch_cmds(args)
        mock_cmds.service_run.assert_called_with(image=args.image, name=args.name, cpu_shares=args.cpushares,
                                                 memory=args.memory, target_num_containers=args.target_num_containers,
                                                 privileged=args.privileged,
                                                 run_command=args.run_command,
                                                 entrypoint=args.entrypoint, expose=args.expose, publish=args.publish,
                                                 envvars=args.env, envfiles=args.env_file, tag=args.tag,
                                                 linked_to_service=args.link_service,
                                                 autorestart=args.autorestart, autodestroy=args.autodestroy,
                                                 autoredeploy=args.autoredeploy, roles=args.role,
                                                 sequential=args.sequential,
                                                 volume=args.volume, volumes_from=args.volumes_from,
                                                 deployment_strategy=args.deployment_strategy)

        args = self.parser.parse_args(['service', 'scale', 'id', '3'])
        dispatch_cmds(args)
        mock_cmds.service_scale.assert_called_with(args.identifier, args.target_num_containers)

        args = self.parser.parse_args(['service', 'set', 'id'])
        dispatch_cmds(args)
        mock_cmds.service_set.assert_called_with(args.identifier, image=args.image, cpu_shares=args.cpushares,
                                                 memory=args.memory, privileged=args.privileged,
                                                 target_num_containers=args.target_num_containers,
                                                 run_command=args.run_command,
                                                 entrypoint=args.entrypoint, expose=args.expose, publish=args.publish,
                                                 envvars=args.env, envfiles=args.env_file,
                                                 tag=args.tag, linked_to_service=args.link_service,
                                                 autorestart=args.autorestart, autodestroy=args.autodestroy,
                                                 autoredeploy=args.autoredeploy, roles=args.role,
                                                 sequential=args.sequential, redeploy=args.redeploy,
                                                 volume=args.volume, volumes_from=args.volumes_from,
                                                 deployment_strategy=args.deployment_strategy)

        args = self.parser.parse_args(['service', 'start', 'id'])
        dispatch_cmds(args)
        mock_cmds.service_start.assert_called_with(args.identifier)

        args = self.parser.parse_args(['service', 'stop', 'id'])
        dispatch_cmds(args)
        mock_cmds.service_stop.assert_called_with(args.identifier)

        args = self.parser.parse_args(['service', 'terminate', 'id'])
        dispatch_cmds(args)
        mock_cmds.service_terminate.assert_called_with(args.identifier)

    @mock.patch('tutumcli.tutum_cli.commands')
    def test_container_dispatch(self, mock_cmds):
        args = self.parser.parse_args(['container', 'inspect', 'id'])
        dispatch_cmds(args)
        mock_cmds.container_inspect.assert_called_with(args.identifier)

        args = self.parser.parse_args(['container', 'logs', 'id'])
        dispatch_cmds(args)
        mock_cmds.container_logs.assert_called_with(args.identifier)

        args = self.parser.parse_args(['container', 'ps'])
        dispatch_cmds(args)
        mock_cmds.container_ps.assert_called_with(args.quiet, args.status, args.service)

        args = self.parser.parse_args(['container', 'start', 'id'])
        dispatch_cmds(args)
        mock_cmds.container_start.assert_called_with(args.identifier)

        args = self.parser.parse_args(['container', 'stop', 'id'])
        dispatch_cmds(args)
        mock_cmds.container_stop.assert_called_with(args.identifier)

        args = self.parser.parse_args(['container', 'terminate', 'id'])
        dispatch_cmds(args)
        mock_cmds.container_terminate.assert_called_with(args.identifier)

    @mock.patch('tutumcli.tutum_cli.commands')
    def test_image_dispatch(self, mock_cmds):
        args = self.parser.parse_args(['image', 'list'])
        dispatch_cmds(args)
        mock_cmds.image_list.assert_called_with(args.quiet, args.jumpstarts, args.linux)

        args = self.parser.parse_args(['image', 'register', 'name'])
        dispatch_cmds(args)
        mock_cmds.image_register(args.image_name, args.description)

        args = self.parser.parse_args(['image', 'push', 'name'])
        dispatch_cmds(args)
        mock_cmds.image_push(args.name, args.public)

        args = self.parser.parse_args(['image', 'rm', 'name'])
        dispatch_cmds(args)
        mock_cmds.image_rm(args.image_name)

        args = self.parser.parse_args(['image', 'search', 'name'])
        dispatch_cmds(args)
        mock_cmds.image_search(args.query)

        args = self.parser.parse_args(['image', 'update', 'name'])
        dispatch_cmds(args)
        mock_cmds.image_update(args.image_name, args.username, args.password, args.description)

    @mock.patch('tutumcli.tutum_cli.commands')
    def test_node_dispatch(self, mock_cmds):
        args = self.parser.parse_args(['node', 'inspect', 'id'])
        dispatch_cmds(args)
        mock_cmds.node_inspect.assert_called_with(args.identifier)

        args = self.parser.parse_args(['node', 'list'])
        dispatch_cmds(args)
        mock_cmds.node_list(args.quiet)

        args = self.parser.parse_args(['node', 'rm', 'id'])
        dispatch_cmds(args)
        mock_cmds.node_rm(args.identifier)

        args = self.parser.parse_args(['node', 'upgrade', 'id'])
        dispatch_cmds(args)
        mock_cmds.node_rm(args.identifier)

    @mock.patch('tutumcli.tutum_cli.commands')
    def test_nodecluster_dispatch(self, mock_cmds):
        args = self.parser.parse_args(['nodecluster', 'create', 'name', '1', '2', '3'])
        dispatch_cmds(args)
        mock_cmds.nodecluster_create(args.target_num_nodes, args.name,
                                     args.provider, args.region, args.nodetype)

        args = self.parser.parse_args(['nodecluster', 'inspect', 'id'])
        dispatch_cmds(args)
        mock_cmds.nodecluster_inspect(args.identifier)

        args = self.parser.parse_args(['nodecluster', 'list'])
        dispatch_cmds(args)
        mock_cmds.nodecluster_list(args.quiet)

        args = self.parser.parse_args(['nodecluster', 'provider'])
        dispatch_cmds(args)
        mock_cmds.nodecluster_show_providers(args.quiet)

        args = self.parser.parse_args(['nodecluster', 'region', '-p', 'digitalocean'])
        dispatch_cmds(args)
        mock_cmds.nodecluster_show_regions(args.provider)

        args = self.parser.parse_args(['nodecluster', 'nodetype', '-r', 'ams1', '-p', 'digitalocean'])
        dispatch_cmds(args)
        mock_cmds.nodecluster_show_types(args.provider, args.region)

        args = self.parser.parse_args(['nodecluster', 'rm', 'id'])
        dispatch_cmds(args)
        mock_cmds.nodecluster_rm(args.identifier)

        args = self.parser.parse_args(['nodecluster', 'scale', 'id', '3'])
        dispatch_cmds(args)
        mock_cmds.nodecluster_scale(args.identifier, args.target_num_nodes)

    @mock.patch('tutumcli.tutum_cli.commands')
    def test_tag_dispatch(self, mock_cmds):
        args = self.parser.parse_args(['tag', 'add', '-t', 'abc', 'id'])
        dispatch_cmds(args)
        mock_cmds.tag_add.assert_called_with(args.identifier, args.tag)

        args = self.parser.parse_args(['tag', 'list', 'abc', 'id'])
        dispatch_cmds(args)
        mock_cmds.tag_list.assert_called_with(args.identifier, args.quiet)

        args = self.parser.parse_args(['tag', 'rm', '-t', 'abc', 'id'])
        dispatch_cmds(args)
        mock_cmds.tag_rm.assert_called_with(args.identifier, args.tag)

        args = self.parser.parse_args(['tag', 'set', '-t', 'abc', 'id'])
        dispatch_cmds(args)
        mock_cmds.tag_set.assert_called_with(args.identifier, args.tag)


class ParserTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO.StringIO()

    def tearDown(self):
        sys.stdout = self.stdout

    def compare_output(self, output, args):
        parser = initialize_parser()
        argv = patch_help_option(args)

        parser.parse_args(argv)

        out = self.buf.getvalue()
        self.buf.truncate(0)

        self.assertEqual(' '.join(output.split()), ' '.join(out.split()))

    @mock.patch('tutumcli.tutum_cli.argparse.ArgumentParser.add_argument')
    @mock.patch('tutumcli.tutum_cli.argparse.ArgumentParser.exit')
    def test_tutum_version(self, mock_exit, mock_add_arg):
        initialize_parser()
        mock_add_arg.assert_any_call('-v', '--version', action='version', version='%(prog)s ' + tutumcli.__version__)



class NonUniqueIdentifier(RuntimeError):
    pass


class ObjectNotFound(RuntimeError):
    pass


class BadParameter(RuntimeError):
    pass


class DockerNotFound(RuntimeError):
    pass


class PublicImageNotFound(RuntimeError):
    pass


class StreamOutputError(Exception):
    pass


class InternalError(RuntimeError):
    pass